local Aloft = Aloft
if not Aloft then return end
local AloftModules = AloftModules
if not AloftModules then return end

-----------------------------------------------------------------------------

AloftModules:AddInitializer("AloftAutoShow", function()

-----------------------------------------------------------------------------

local AloftAutoShow = Aloft:NewModule("AutoShow", "AceEvent-2.0", "AceHook-2.1")
AloftAutoShow.dynamic = true

-----------------------------------------------------------------------------

AloftAutoShow.db = Aloft:AcquireDBNamespace("autoShow")
Aloft:RegisterDefaults("autoShow", "profile", {
	allEnemyEnable	= true,
	allFriendEnable	= false,

	combatEnemyEnable	= true,
	combatFriendEnable	= false,
	flaggedEnemyEnable	= true,
	flaggedFriendEnable	= false,
	groupEnemyEnable	= true,
	groupFriendEnable	= false,
	restingEnemyEnable	= true,
	restingFriendEnable	= false,
	defaultEnemyEnable	= true,
	defaultFriendEnable	= false,

	worldEnemyEnable	= true,
	worldFriendEnable	= false,
	battlegroundEnemyEnable	= true,
	battlegroundFriendEnable	= false,
	arenaEnemyEnable	= true,
	arenaFriendEnable	= false,
	partyEnemyEnable	= true,
	partyFriendEnable	= false,
	raidEnemyEnable	= true,
	raidFriendEnable	= false,
})


-----------------------------------------------------------------------------

local profile

local IsInInstance = IsInInstance
local IsResting = IsResting
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers
local UnitAffectingCombat = UnitAffectingCombat
local UnitIsPVP = UnitIsPVP
local UnitIsPVPFreeForAll = UnitIsPVPFreeForAll

local unpack = unpack

-----------------------------------------------------------------------------

function AloftAutoShow:OnInitialize()
	profile = self.db.profile
end

function AloftAutoShow:OnEnable()
	self:RegisterEvents()
	self:HookNameplates()

	-- turn on/off namplates based on current state
	self:ApplyNameplateDisplay()
end

function AloftAutoShow:OnDisable()
	self:UnregisterAllEvents()
	self:UnhookAll()
end

-----------------------------------------------------------------------------

function AloftAutoShow:RegisterEvents()
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "OnPlayerRegenDisabled")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnPlayerRegenEnabled")
	self:RegisterEvent("PLAYER_UPDATE_RESTING", "OnPlayerUpdateResting")
	self:RegisterEvent("RAID_ROSTER_UPDATE", "OnRaidRosterUpdate")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "OnPartyMembersChanged")
	-- self:RegisterEvent("ARENA_TEAM_ROSTER_UPDATE", "OnArenaTeamRosterUpdate") -- TODO: determine if this is even needed; this fires repeatedly when things change; don't know what to do about it
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA","OnPlayerEnteringWorld")
end

function AloftAutoShow:HookNameplates()
	self:UnhookAll()

	self:Hook(Aloft, "SetEnemyNameplateDisplay")
	self:Hook(Aloft, "SetFriendNameplateDisplay")
end

-----------------------------------------------------------------------------

function AloftAutoShow:OnPlayerRegenDisabled()
	self:ApplyNameplateDisplay()
end

function AloftAutoShow:OnPlayerRegenEnabled()
	self:ApplyNameplateDisplay()
end

function AloftAutoShow:OnPlayerUpdateResting()
	self:ApplyNameplateDisplay()
end

function AloftAutoShow:OnRaidRosterUpdate()
	self:ApplyNameplateDisplay()
end

function AloftAutoShow:OnPartyMembersChanged()
	self:ApplyNameplateDisplay()
end

function AloftAutoShow:OnArenaTeamRosterUpdate()
	self:ApplyNameplateDisplay()
end

function AloftAutoShow:OnPlayerEnteringWorld()
	self:ApplyNameplateDisplay()
end

-----------------------------------------------------------------------------

function AloftAutoShow:GetEnemyNameplateDisplay()
	local stateValue
	if UnitAffectingCombat("player") then
		stateValue = profile.combatEnemyEnable
	elseif UnitIsPVP("player") or UnitIsPVPFreeForAll("player") then
		stateValue = profile.flaggedEnemyEnable
	elseif IsResting() then
		stateValue = profile.restingEnemyEnable
	elseif (GetNumPartyMembers() > 0)  or (GetNumRaidMembers() > 0) then
		stateValue = profile.groupEnemyEnable
	else
		stateValue = profile.defaultEnemyEnable
	end

	local placeValue
	local inInstance, instanceType = IsInInstance()
	if inInstance and instanceType ~= "none"then
		if instanceType == "pvp" then
			placeValue = profile.battlegroundEnemyEnable
		elseif instanceType == "arena" then
			placeValue = profile.arenaEnemyEnable
		elseif instanceType == "party" then
			placeValue = profile.partyEnemyEnable
		elseif instanceType == "raid" then
			placeValue = profile.raidEnemyEnable
		else
			placeValue = profile.worldEnemyEnable
		end
	else
		placeValue = profile.worldEnemyEnable
	end

	return stateValue and placeValue
end

function AloftAutoShow:SetEnemyNameplateDisplay(this, value)
	if profile.allEnemyEnable then
		self:SetAllEnemyNameplateDisplay(value)
	else
		if UnitAffectingCombat("player") then
			profile.combatEnemyEnable = value
		elseif UnitIsPVP("player") or UnitIsPVPFreeForAll("player") then
			profile.flaggedEnemyEnable = value
		elseif IsResting() then
			profile.restingEnemyEnable = value
		elseif (GetNumPartyMembers() > 0)  or (GetNumRaidMembers() > 0) then
			profile.groupEnemyEnable = value
		else
			profile.defaultEnemyEnable = value
		end

		local inInstance, instanceType = IsInInstance()
		if inInstance and instanceType ~= "none"then
			if instanceType == "pvp" then
				profile.battlegroundEnemyEnable = value
			elseif instanceType == "arena" then
				profile.arenaEnemyEnable = value
			elseif instanceType == "party" then
				profile.partyEnemyEnable = value
			elseif instanceType == "raid" then
				profile.raidEnemyEnable = value
			else
				profile.worldEnemyEnable = value
			end
		else
			profile.worldEnemyEnable = value
		end
	end

	-- self.hooks[this].SetEnemyNameplateDisplay(this, value)
	Aloft:SetNameplateCVar("nameplateShowEnemies", value)
end

function AloftAutoShow:SetAllEnemyNameplateDisplay(value)
	profile.combatEnemyEnable = value
	profile.flaggedEnemyEnable = value
	profile.restingEnemyEnable = value
	profile.groupEnemyEnable = value
	profile.defaultEnemyEnable = value

	profile.worldEnemyEnable = value
	profile.battlegroundEnemyEnable = value
	profile.arenaEnemyEnable = value
	profile.partyEnemyEnable = value
	profile.raidEnemyEnable = value
end

-----------------------------------------------------------------------------

function AloftAutoShow:GetFriendNameplateDisplay()
	local stateValue
	if UnitAffectingCombat("player") then
		stateValue = profile.combatFriendEnable
	elseif UnitIsPVP("player") or UnitIsPVPFreeForAll("player") then
		stateValue = profile.flaggedFriendEnable
	elseif IsResting() then
		stateValue = profile.restingFriendEnable
	elseif (GetNumPartyMembers() > 0)  or (GetNumRaidMembers() > 0) then
		stateValue = profile.groupFriendEnable
	else
		stateValue = profile.defaultFriendEnable
	end

	local placeValue
	local inInstance, instanceType = IsInInstance()
	if inInstance and instanceType ~= "none"then
		if instanceType == "pvp" then
			placeValue = profile.battlegroundFriendEnable
		elseif instanceType == "arena" then
			placeValue = profile.arenaFriendEnable
		elseif instanceType == "party" then
			placeValue = profile.partyFriendEnable
		elseif instanceType == "raid" then
			placeValue = profile.raidFriendEnable
		else
			placeValue = profile.worldFriendEnable
		end
	else
		placeValue = profile.worldFriendEnable
	end

	return stateValue and placeValue
end

function AloftAutoShow:SetFriendNameplateDisplay(this, value)
	if profile.allFriendEnable then
		self:SetAllFriendNameplateDisplay(value)
	else
		if UnitAffectingCombat("player") then
			profile.combatFriendEnable = value
		elseif UnitIsPVP("player") or UnitIsPVPFreeForAll("player") then
			profile.flaggedFriendEnable = value
		elseif IsResting() then
			profile.restingFriendEnable = value
		elseif (GetNumPartyMembers() > 0)  or (GetNumRaidMembers() > 0) then
			profile.groupFriendEnable = value
		else
			profile.defaultFriendEnable = value
		end
	end

	local inInstance, instanceType = IsInInstance()
	if inInstance and instanceType ~= "none"then
		if instanceType == "pvp" then
			profile.battlegroundFriendEnable = value
		elseif instanceType == "arena" then
			profile.arenaFriendEnable = value
		elseif instanceType == "party" then
			profile.partyFriendEnable = value
		elseif instanceType == "raid" then
			profile.raidFriendEnable = value
		else
			profile.worldFriendEnable = value
		end
	else
		profile.worldFriendEnable = value
	end

	-- self.hooks[this].SetFriendNameplateDisplay(this, value)
	Aloft:SetNameplateCVar("nameplateShowFriends", value)
end

function AloftAutoShow:SetAllFriendNameplateDisplay(value)
	profile.combatFriendEnable = value
	profile.flaggedFriendEnable = value
	profile.restingFriendEnable = value
	profile.groupFriendEnable = value
	profile.defaultFriendEnable = value

	profile.worldFriendEnable = value
	profile.battlegroundFriendEnable = value
	profile.arenaFriendEnable = value
	profile.partyFriendEnable = value
	profile.raidFriendEnable = value
end

-----------------------------------------------------------------------------

function AloftAutoShow:ApplyNameplateDisplay()
	Aloft:SetNameplateCVar("nameplateShowEnemies", self:GetEnemyNameplateDisplay())
	Aloft:SetNameplateCVar("nameplateShowFriends", self:GetFriendNameplateDisplay())
end

-----------------------------------------------------------------------------

end)
