local Aloft = Aloft
if not Aloft then return end
local AloftModules = AloftModules
if not AloftModules then return end

-----------------------------------------------------------------------------

AloftModules:AddInitializer("AloftAutoShow", function()

-----------------------------------------------------------------------------

if not Aloft:HasModule("AutoShow") then return end

local AloftAutoShow = Aloft:GetModule("AutoShow")
local L = AceLibrary("AceLocale-2.2"):new("AloftAutoShowOptions")

-----------------------------------------------------------------------------

Aloft.Options.args.show.type = 'group'
Aloft.Options.args.show.get = nil
Aloft.Options.args.show.set = nil
Aloft.Options.args.show.args =
{
	all =
	{
		type = 'toggle',
		name = L["Set All"],
		desc = L["Auto-show/hide all neutral and hostile nameplates (subject to Visibility options)"],
		order = -1,
		get = function() return AloftAutoShow.db.profile.allEnemyEnable end,
		set = function(v) AloftAutoShow.db.profile.allEnemyEnable = v AloftAutoShow:SetAllEnemyNameplateDisplay(v) AloftAutoShow:ApplyNameplateDisplay() end,
	},
	state =
	{
		type = "group",
		name = L["Player State"],
		desc = L["Auto-show/hide all neutral and hostile nameplates based on player state (subject to Visibility options)"],
		args =
		{
			combat =
			{
				type = 'toggle',
				name = L["Combat"],
				desc = L["Auto-show/hide all neutral and hostile nameplates on entering/exiting combat (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.combatEnemyEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allEnemyEnable then AloftAutoShow:SetAllEnemyNameplateDisplay(v) else AloftAutoShow.db.profile.combatEnemyEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			flagged =
			{
				type = 'toggle',
				name = L["Flagged/PvP"],
				desc = L["Auto-show/hide all neutral and hostile nameplates on flagging/unflagging for PvP (and not in combat; subject to Visibility options)"],
				order = 2,
				get = function() return AloftAutoShow.db.profile.flaggedEnemyEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allEnemyEnable then AloftAutoShow:SetAllEnemyNameplateDisplay(v) else AloftAutoShow.db.profile.flaggedEnemyEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			resting =
			{
				type = 'toggle',
				name = L["Resting"],
				desc = L["Auto-show/hide all neutral and hostile nameplates on entering/exiting resting state (and not in combat or flagged; subject to Visibility options)"],
				order = 3,
				get = function() return AloftAutoShow.db.profile.restingEnemyEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allEnemyEnable then AloftAutoShow:SetAllEnemyNameplateDisplay(v) else AloftAutoShow.db.profile.restingEnemyEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			group =
			{
				type = 'toggle',
				name = L["Group"],
				desc = L["Auto-show/hide all neutral and hostile nameplates on entering/exiting groups (and not in combat, flagged, or resting; subject to Visibility options)"],
				order = 4,
				get = function() return AloftAutoShow.db.profile.groupEnemyEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allEnemyEnable then AloftAutoShow:SetAllEnemyNameplateDisplay(v) else AloftAutoShow.db.profile.groupEnemyEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			default =
			{
				type = 'toggle',
				name = L["Default"],
				desc = L["Auto-show/hide all neutral and hostile nameplates by default (subject to Visibility options)"],
				order = 5,
				get = function() return AloftAutoShow.db.profile.defaultEnemyEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allEnemyEnable then AloftAutoShow:SetAllEnemyNameplateDisplay(v) else AloftAutoShow.db.profile.defaultEnemyEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
		},
	},
	location =
	{
		type = "group",
		name = L["Player Location"],
		desc = L["Auto-show/hide all neutral and hostile nameplates based on player location (subject to Visibility options)"],
		args =
		{
			world =
			{
				type = 'toggle',
				name = L["World"],
				desc = L["Auto-show/hide all neutral and hostile nameplates on entering/exiting the world (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.worldEnemyEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allEnemyEnable then AloftAutoShow:SetAllEnemyNameplateDisplay(v) else AloftAutoShow.db.profile.worldEnemyEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			battleground =
			{
				type = 'toggle',
				name = L["Battleground"],
				desc = L["Auto-show/hide all neutral and hostile nameplates on entering/exiting battlegrounds (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.battlegroundEnemyEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allEnemyEnable then AloftAutoShow:SetAllEnemyNameplateDisplay(v) else AloftAutoShow.db.profile.battlegroundEnemyEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			arena =
			{
				type = 'toggle',
				name = L["Arena"],
				desc = L["Auto-show/hide all neutral and hostile nameplates on entering/exiting arenas (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.arenaEnemyEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allEnemyEnable then AloftAutoShow:SetAllEnemyNameplateDisplay(v) else AloftAutoShow.db.profile.arenaEnemyEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			party =
			{
				type = 'toggle',
				name = L["5-Man Instance"],
				desc = L["Auto-show/hide all neutral and hostile nameplates on entering/exiting 5-man instances (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.partyEnemyEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allEnemyEnable then AloftAutoShow:SetAllEnemyNameplateDisplay(v) else AloftAutoShow.db.profile.partyEnemyEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			raid =
			{
				type = 'toggle',
				name = L["Raid Instance"],
				desc = L["Auto-show/hide all neutral and hostile nameplates on entering/exiting raid instances (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.raidEnemyEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allEnemyEnable then AloftAutoShow:SetAllEnemyNameplateDisplay(v) else AloftAutoShow.db.profile.raidEnemyEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
		},
	},
}

Aloft.Options.args.showFriendly.type = 'group'
Aloft.Options.args.showFriendly.get = nil
Aloft.Options.args.showFriendly.set = nil
Aloft.Options.args.showFriendly.args =
{
	all =
	{
		type = 'toggle',
		name = L["Set All"],
		desc = L["Auto-show/hide all friendly nameplates (subject to Visibility options)"],
		order = -1,
		get = function() return AloftAutoShow.db.profile.allFriendEnable end,
		set = function(v) AloftAutoShow.db.profile.allFriendEnable = v AloftAutoShow:SetAllFriendNameplateDisplay(v) AloftAutoShow:ApplyNameplateDisplay() end,
	},
	state =
	{
		type = "group",
		name = L["Player State"],
		desc = L["Auto-show/hide all friendly nameplates based on player state (subject to Visibility options)"],
		args =
		{
			combat =
			{
				type = 'toggle',
				name = L["Combat"],
				desc = L["Auto-show/hide all friendly nameplates on entering/exiting combat (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.combatFriendEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allFriendEnable then AloftAutoShow:SetAllFriendNameplateDisplay(v) else AloftAutoShow.db.profile.combatFriendEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			flagged =
			{
				type = 'toggle',
				name = L["Flagged/PvP"],
				desc = L["Auto-show/hide all friendly nameplates on flagging/unflagging for PvP (and not in combat; subject to Visibility options)"],
				order = 2,
				get = function() return AloftAutoShow.db.profile.flaggedFriendEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allFriendEnable then AloftAutoShow:SetAllFriendNameplateDisplay(v) else AloftAutoShow.db.profile.flaggedFriendEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			resting =
			{
				type = 'toggle',
				name = L["Resting"],
				desc = L["Auto-show/hide all friendly nameplates on entering/exiting resting state (and not in combat or flagged; subject to Visibility options)"],
				order = 3,
				get = function() return AloftAutoShow.db.profile.restingFriendEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allFriendEnable then AloftAutoShow:SetAllFriendNameplateDisplay(v) else AloftAutoShow.db.profile.restingFriendEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			group =
			{
				type = 'toggle',
				name = L["Group"],
				desc = L["Auto-show/hide all friendly nameplates on entering/exiting groups (and not in combat, flagged, or resting; subject to Visibility options)"],
				order = 4,
				get = function() return AloftAutoShow.db.profile.groupFriendEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allFriendEnable then AloftAutoShow:SetAllFriendNameplateDisplay(v) else AloftAutoShow.db.profile.groupFriendEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			default =
			{
				type = 'toggle',
				name = L["Default"],
				desc = L["Auto-show/hide all friendly nameplates by default (subject to Visibility options)"],
				order = 5,
				get = function() return AloftAutoShow.db.profile.defaultFriendEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allFriendEnable then AloftAutoShow:SetAllFriendNameplateDisplay(v) else AloftAutoShow.db.profile.defaultFriendEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
		},
	},
	location =
	{
		type = "group",
		name = L["Player Location"],
		desc = L["Auto-show/hide all friendly nameplates based on player location (subject to Visibility options)"],
		args =
		{
			world =
			{
				type = 'toggle',
				name = L["World"],
				desc = L["Auto-show/hide all friendly nameplates on entering/exiting the world (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.worldFriendEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allFriendEnable then AloftAutoShow:SetAllFriendNameplateDisplay(v) else AloftAutoShow.db.profile.worldFriendEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			battleground =
			{
				type = 'toggle',
				name = L["Battleground"],
				desc = L["Auto-show/hide all friendly nameplates on entering/exiting battlegrounds (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.battlegroundFriendEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allFriendEnable then AloftAutoShow:SetAllFriendNameplateDisplay(v) else AloftAutoShow.db.profile.battlegroundFriendEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			arena =
			{
				type = 'toggle',
				name = L["Arena"],
				desc = L["Auto-show/hide all friendly nameplates on entering/exiting arenas (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.arenaFriendEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allFriendEnable then AloftAutoShow:SetAllFriendNameplateDisplay(v) else AloftAutoShow.db.profile.arenaFriendEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			party =
			{
				type = 'toggle',
				name = L["5-Man Instance"],
				desc = L["Auto-show/hide all friendly nameplates on entering/exiting 5-man instances (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.partyFriendEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allFriendEnable then AloftAutoShow:SetAllFriendNameplateDisplay(v) else AloftAutoShow.db.profile.partyFriendEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
			raid =
			{
				type = 'toggle',
				name = L["Raid Instance"],
				desc = L["Auto-show/hide all friendly nameplates on entering/exiting raid instances (subject to Visibility options)"],
				order = 1,
				get = function() return AloftAutoShow.db.profile.raidFriendEnable end,
				set = function(v)
					if AloftAutoShow.db.profile.allFriendEnable then AloftAutoShow:SetAllFriendNameplateDisplay(v) else AloftAutoShow.db.profile.raidFriendEnable = v end
					AloftAutoShow:ApplyNameplateDisplay()
				end,
			},
		},
	},
}

-----------------------------------------------------------------------------

end)
