local Aloft = Aloft
if not Aloft then return end
local AloftModules = AloftModules
if not AloftModules then return end

-----------------------------------------------------------------------------

AloftModules:AddInitializer("AloftCombatText", function()

-----------------------------------------------------------------------------

local AloftCombatText = Aloft:NewModule("CombatText", "AceEvent-2.0")
AloftCombatText.dynamic = true

local SML = AceLibrary("LibSharedMedia-3.0")

-----------------------------------------------------------------------------

AloftCombatText.db = Aloft:AcquireDBNamespace("combatText")
Aloft:RegisterDefaults("combatText", "profile", {
	enable			= true, -- not used for any module control logic, just required for PlaceFrame and PrepareText to work
	enableGroup		= false,
	enableTarget	= false,
	alpha			= 0.75,
	point			= "CENTER",
	relativeToPoint	= "CENTER",
	offsetX			= 0,
	offsetY			= 0,
	font			= "Friz Quadrata TT",
	fontSize		= 16,
	shadow			= false,
	outline			= "OUTLINE",
	colors =
	{
		--[[ -- original... UNIT_COMBAT event was updated with flag-style school type at some unknown point in time
		[0] = { 1, 1, 1 },			-- Physical
		[1] = { 1, 1, 0.5 },  		-- Holy
		[2] = { 1, 0.3, 0 },		-- Fire
		[3] = { 0.5, 1, 0.2 },		-- Nature
		[4] = { 0.4, 0.6, 0.9 },	-- Frost,
		[5] = { 0.7, 0.4, 1 },		-- Shadow
		[6] = { 0.8, 0.8, 1 },		-- Arcane
		]]

		[1]  = { 1, 1, 1 },			-- Physical
		[2]  = { 1, 1, 0.5 },  		-- Holy
		[4]  = { 1, 0.3, 0 },			-- Fire
		[8]  = { 0.5, 1, 0.2 },		-- Nature
		[16] = { 0.4, 0.6, 0.9 },		-- Frost,
		[32] = { 0.7, 0.4, 1 },		-- Shadow
		[64] = { 0.8, 0.8, 1 },		-- Arcane

		energize = { 0.41, 0.8, 0.94 },
		text = { 1, 1, 1 },
		heal = { 0, 1, 0 },
		default = { 0.5, 0.5, 0.5 },
	},
})

-----------------------------------------------------------------------------

-- local textRegionPool = { }
local updateList = { }
local profile
local colors

-----------------------------------------------------------------------------

function AloftCombatText:RegisterEvents()
	if profile and (profile.enableGroup or profile.enableTarget) then
		self:RegisterEvent("PLAYER_TARGET_CHANGED", "OnPlayerTargetChanged")
		-- self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "OnCombatLogEvent")
		self:RegisterEvent("UNIT_COMBAT", "OnUnitCombat")

		self:RegisterEvent("Aloft:OnNameplateHide", "OnNameplateHide")
	else
		self:CancelEvent("PLAYER_TARGET_CHANGED")
		-- self:CancelEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:CancelEvent("UNIT_COMBAT")

		self:CancelEvent("Aloft:OnNameplateHide")
	end
end

function AloftCombatText:RequiresData()
	if profile and profile.enableGroup then
		return "unitid"
	end
end

-----------------------------------------------------------------------------

function AloftCombatText:UpdateAll()
	for aloftData in Aloft:IterateNameplates() do
		if profile and (profile.enableTarget or profile.enableGroup) then
			local layoutFrame = aloftData.layoutFrame
			if layoutFrame then
				local combatTextRegion = layoutFrame.combatTextRegion
				if combatTextRegion then
					self:PlaceFrame(combatTextRegion, layoutFrame, profile)
					self:PrepareText(combatTextRegion, profile)
				end
			end
		else
			self:OnNameplateHide(aloftData)
		end
	end
	for layoutFrame in pairs(Aloft.layoutFramePool) do
		local combatTextRegion = layoutFrame.combatTextRegion
		if combatTextRegion then
			self:PrepareText(combatTextRegion, profile)
			self:PlaceFrame(combatTextRegion, layoutFrame, profile)
		end
	end
	self:RegisterEvents()
end

function AloftCombatText:Update()
	local now = GetTime()

	for aloftData, startTime in pairs(updateList) do
		local layoutFrame = aloftData.layoutFrame
		local combatTextRegion = layoutFrame and layoutFrame.combatTextRegion

		if combatTextRegion then
			local elapsedTime = now - startTime
			if elapsedTime < COMBATFEEDBACK_FADEINTIME then
				combatTextRegion:SetAlpha(profile.alpha*(elapsedTime / COMBATFEEDBACK_FADEINTIME))
			elseif elapsedTime < (COMBATFEEDBACK_FADEINTIME + COMBATFEEDBACK_HOLDTIME) then
				combatTextRegion:SetAlpha(profile.alpha)
			elseif elapsedTime < (COMBATFEEDBACK_FADEINTIME + COMBATFEEDBACK_HOLDTIME + COMBATFEEDBACK_FADEOUTTIME) then
				combatTextRegion:SetAlpha(profile.alpha * (1 - (elapsedTime - COMBATFEEDBACK_HOLDTIME - COMBATFEEDBACK_FADEINTIME) / COMBATFEEDBACK_FADEOUTTIME))
			else
				combatTextRegion:Hide()
				updateList[aloftData] = nil
			end
		end
	end

	if not next(updateList) then
		self:CancelScheduledEvent("AloftCombatText:Update")
	end
end

-----------------------------------------------------------------------------

function AloftCombatText:OnInitialize()
	profile = self.db.profile
	colors = profile.colors
end

function AloftCombatText:OnEnable()
	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if profile[type] then
			profile[type] = value
			self:UpdateAll()
		end
	end)

	self:UpdateAll()
end

function AloftCombatText:OnDisable()
	self:UnregisterAllEvents()

	for aloftData in Aloft:IterateVisibleNameplates() do
		self:OnNameplateHide(aloftData)
	end
end

-----------------------------------------------------------------------------

local playerTargetGUId = nil
function AloftCombatText:OnPlayerTargetChanged()
	playerTargetGUId = UnitGUID("target")
	-- ChatFrame7:AddMessage("AloftCombatText:OnPlayerTargetChanged(): " .. tostring(UnitName("target")) .. "/" .. tostring(playerTargetGUId))
end

function AloftCombatText:OnNameplateHide(aloftData)
	local layoutFrame = aloftData.layoutFrame
	local combatTextRegion = layoutFrame and layoutFrame.combatTextRegion
	if combatTextRegion then
		combatTextRegion:Hide()
		combatTextRegion:SetText("")

		if not next(updateList) then
			self:CancelScheduledEvent("AloftCombatText:Update")
		end
	end
end

-----------------------------------------------------------------------------

-- These return text, scale, r, g, b
local combatHandlers =
{
	["BLOCK"]		= function() return CombatFeedbackText["BLOCK"], 0.75, colors.text end,
	["ENERGIZE"]	= function(flags, amount) return amount, flags == "CRITICAL" and 1.3 or 1.0, colors.energize end,
	["HEAL"]		= function(flags, amount) return "+"..amount, flags == "CRITICAL" and 1.3 or 1.0, colors.heal end,
	["IMMUNE"]		= function() return CombatFeedbackText["IMMUNE"], 0.5, colors.text end,
	["WOUND"]		= function(flags, amount, typ)
						if amount ~= 0 then
							local scale = 1
							if flags == "CRITICAL" or flags == "CRUSHING" then
								scale = 1.5
							elseif flags == "GLANCING" then
								scale = 0.75
							end
							return "-"..amount, scale, colors[typ]
						elseif flags == "ABSORB" or flags == "BLOCK" or flags == "RESIST" then
							return CombatFeedbackText[flags], 0.75, colors.text
						end
						return CombatFeedbackText[flags], 1.0, colors.text
					  end,
}

-----------------------------------------------------------------------------

function AloftCombatText:OnUnitCombat(unit, event, ...) -- last 3 arguments are flags, amount, type
	if unit == "player" then return end

	local aloftData = self:GetUnitNameplate(unit)
	if not aloftData then return end

	local text, scale, color
	if combatHandlers[event] then
		text, scale, color = combatHandlers[event](...)
	else
		text, scale, color = CombatFeedbackText[event], 1.0, colors.text
	end

	self:PostCombatText(aloftData, text, scale, color)
	self:DumpOnUnitCombat(color or colors.default, unit, event, ...)
end

-----------------------------------------------------------------------------

function AloftCombatText:GetUnitNameplate(unit)
	local aloftData = nil
	if unit == "target" then
		if not profile.enableTarget then return end
		if profile.enableGroup and (UnitInParty(unit) or UnitInRaid(unit)) then return end
		aloftData = Aloft:GetTargetNameplate()
	else
		if not profile.enableGroup then return end
		aloftData = Aloft:GetUnitNameplate(unit)
	end
	return aloftData
end

function AloftCombatText:GetGUIDNameplate(unit)
	local aloftData = nil
	return aloftData
end

function AloftCombatText:PostCombatText(aloftData, text, scale, color)
	local layoutFrame = aloftData.layoutFrame
	if not layoutFrame then
		layoutFrame = Aloft:AcquireLayoutFrame(aloftData)
	end
	local combatTextRegion = layoutFrame.combatTextRegion

	if not combatTextRegion then
		combatTextRegion = aloftData:CreateFontString()

		layoutFrame.combatTextRegion = combatTextRegion

		self:PlaceFrame(combatTextRegion, layoutFrame, profile)
	end

	-- The line below used to crash the Windows WoW client sporadically (not the Mac one). A simple SetTextHeight was used instead.
	combatTextRegion:SetFont(SML:Fetch("font", profile.font), floor((profile.fontSize * scale) + 0.5), profile.outline)
	-- combatTextRegion:SetTextHeight(profile.fontSize * scale)

	combatTextRegion:SetText(text)
	combatTextRegion:SetTextColor(unpack(color or colors.default))
	combatTextRegion:SetAlpha(0)
	combatTextRegion:Show()

	updateList[aloftData] = GetTime()
	self:ScheduleRepeatingEvent("AloftCombatText:Update", self.Update, 0.05, self)
	-- ChatFrame7:AddMessage("AloftCombatText:PostCombatText(): show " .. tostring(aloftData.name) .. "/" .. tostring(text) .. "/" .. tostring(layoutFrame.combatTextRegion:GetText()))
	return
end

-----------------------------------------------------------------------------

local function formatSchool(str, school)
	local color = colors[school] or color.text
	return ("|cff%02x%02x%02x%s|r"):format(floor(color[1]*255), floor(color[2]*255), floor(color[3]*255), tostring(str))
end

local function formatDamage(str, resisted, blocked, absorbed, critical, glancing, crushing)
	local r
	local g
	local b
	local s
	if glancing then
		-- gray
		r = 127
		g = 127
		b = 127
		s = "GLANCING"
	elseif resisted then
		-- green
		r = 0
		g = 255
		b = 0
		s = "RESIST"
	elseif blocked then
		-- blue
		r = 0
		g = 0
		b = 255
		s = "BLOCK"
	elseif absorbed then
		-- cyan
		r = 0
		g = 255
		b = 255
		s = "ABSORB"
	elseif critical then
		-- magenta
		r = 255
		g = 0
		b = 255
		s = "CRITICAL"
	elseif crushing then
		-- yellow
		r = 255
		g = 255
		b = 0
		s = "CRUSHING"
	else
		-- white
		r = 255
		g = 255
		b = 255
		s = "NORMAL"
	end

	return ("|cff%02x%02x%02x%s|r"):format(r, g, b, tostring(str) .. "/" .. s)
end

local function formatMissed(str, missType)
	return ("|cff%02x%02x%02x%s|r"):format(255, 0, 0, tostring(str) .. "/" .. tostring(CombatFeedbackText[missType]))
end

-----------------------------------------------------------------------------

-- ("|cff%02x%02x%02x%s|r"):format(floor(a*255), floor(r*255), floor(g*255), floor(b*255), str)

-----------------------------------------------------------------------------

local function swing_damage(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	ChatFrame7:AddMessage("AloftCombatText:swing_damage(): " .. tostring(originGUId) .. "/" .. tostring(origainName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName)
		.. "/" .. formatSchool(tostring(school), school) .. "/" .. formatDamage(tostring(amount) .. "/" .. tostring(overkill), resisted, blocked, absorbed, critical, glancing, crushing))
end

local function swing_missed(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, missType, amountMissed)
	ChatFrame7:AddMessage("AloftCombatText:swing_missed(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName)
		.. "/" .. formatMissed(tostring(missType) .. "/" .. tostring(amountMissed), missType))
end

-----------------------------------------------------------------------------

local function range_damage(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, spellId, spellName, spellSchool, spellDamage, spellOverkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	ChatFrame7:AddMessage("AloftCombatText:range_damage(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. tostring(spellId) .. "/" .. tostring(spellName) .. "/" .. formatSchool(tostring(spellSchool), spellSchool)
		.. "/" .. formatSchool(tostring(school), school) .. "/" .. formatDamage(tostring(spellDamage) .. "/" .. tostring(spellOverkill), resisted, blocked, absorbed, critical, glancing, crushing))
end

local function range_missed(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, spellId, spellName, spellSchool, missType, amountMissed)
	ChatFrame7:AddMessage("AloftCombatText:range_missed(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. formatSchool(tostring(spellId) .. "/" .. tostring(spellName) .. "/" .. tostring(spellSchool), spellSchool)
		.. "/" .. formatMissed(tostring(missType) .. "/" .. tostring(amountMissed), missType))
end

-----------------------------------------------------------------------------

local function spell_damage(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, spellId, spellName, spellSchool, spellDamage, spellOverkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	ChatFrame7:AddMessage("AloftCombatText:spell_damage(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. formatSchool(tostring(spellId) .. "/" .. tostring(spellName) .. "/" .. tostring(spellSchool), spellSchool)
		.. "/" .. formatSchool(tostring(school), school) .. "/" .. formatDamage(tostring(spellDamage) .. "/" .. tostring(spellOverkill), resisted, blocked, absorbed, critical, glancing, crushing))
end

local function spell_heal(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, spellId, spellName, spellSchool, amount, overhealing, critical)
	ChatFrame7:AddMessage("AloftCombatText:spell_heal(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. formatSchool(tostring(spellId) .. "/" .. tostring(spellName) .. "/" .. tostring(spellSchool), spellSchool)
		.. "/" .. tostring(amount) .. "/" .. tostring(overhealing) .. "/" .. tostring(critical))
end

local function spell_missed(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, spellId, spellName, spellSchool, missType, amountMissed)
	ChatFrame7:AddMessage("AloftCombatText:spell_missed(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. formatSchool(tostring(spellId) .. "/" .. tostring(spellName) .. "/" .. tostring(spellSchool), spellSchool)
		.. "/" .. formatMissed(tostring(missType) .. "/" .. tostring(amountMissed), missType))
end

-----------------------------------------------------------------------------

local function spell_periodic_damage(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, spellId, spellName, spellSchool, spellDamage, spellOverkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	ChatFrame7:AddMessage("AloftCombatText:spell_periodic_damage(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. formatSchool(tostring(spellId) .. "/" .. tostring(spellName) .. "/" .. tostring(spellSchool), spellSchool)
		.. "/" .. formatSchool(tostring(school), school) .. "/" .. formatDamage(tostring(spellDamage) .. "/" .. tostring(spellOverkill), resisted, blocked, absorbed, critical, glancing, crushing))
end

local function spell_periodic_heal(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, spellId, spellName, spellSchool, amount, overhealing, critical)
	ChatFrame7:AddMessage("AloftCombatText:spell_periodic_heal(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. formatSchool(tostring(spellId) .. "/" .. tostring(spellName) .. "/" .. tostring(spellSchool), spellSchool)
		.. "/" .. tostring(amount) .. "/" .. tostring(overhealing) .. "/" .. tostring(critical))
end

local function spell_periodic_missed(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, spellId, spellName, spellSchool, missType, amountMissed)
	ChatFrame7:AddMessage("AloftCombatText:spell_periodic_missed(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. formatSchool(tostring(spellId) .. "/" .. tostring(spellName) .. "/" .. tostring(spellSchool), spellSchool)
		.. "/" .. formatMissed(tostring(missType) .. "/" .. tostring(amountMissed), missType))
end

-----------------------------------------------------------------------------

local function damage_shield(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, spellId, spellName, spellSchool, spellDamage, spellOverkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	ChatFrame7:AddMessage("AloftCombatText:damage_shield(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. formatSchool(tostring(spellId) .. "/" .. tostring(spellName) .. "/" .. tostring(spellSchool), spellSchool)
		.. "/" .. formatSchool(tostring(school), school) .. "/" .. formatDamage(tostring(spellDamage) .. "/" .. tostring(spellOverkill), resisted, blocked, absorbed, critical, glancing, crushing))
end

local function damage_shield_missed(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, spellId, spellName, spellSchool, missType, amountMissed)
	ChatFrame7:AddMessage("AloftCombatText:damage_shield_missed(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. formatSchool(tostring(spellId) .. "/" .. tostring(spellName) .. "/" .. tostring(spellSchool), spellSchool)
		.. "/" .. formatMissed(tostring(missType) .. "/" .. tostring(amountMissed), missType))
end

-----------------------------------------------------------------------------

-- TODO: cannot find what generates this event, some commentary on the web indicates that it is commented out in the Blizzard combat log, should research this
local function damage_split(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, spellId, spellName, spellSchool, spellDamage, spellOverkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	ChatFrame7:AddMessage("AloftCombatText:damage_split(): " .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. formatSchool(tostring(spellId) .. "/" .. tostring(spellName) .. "/" .. tostring(spellSchool), spellSchool)
		.. "/" .. formatSchool(tostring(school), school) .. "/" .. formatDamage(tostring(spellDamage) .. "/" .. tostring(spellOverkill), resisted, blocked, absorbed, critical, glancing, crushing))
end

-----------------------------------------------------------------------------

local subtypeDispatchMethods =
{
	-- prefix: SWING_
	["SWING_DAMAGE"]			= swing_damage,
	["SWING_MISSED"]			= swing_missed,

	-- prefix: RANGE_
	["RANGE_DAMAGE"]			= range_damage,
	["RANGE_MISSED"]			= range_missed,

	-- prefix: SPELL_
	["SPELL_DAMAGE"]			= spell_damage,
	["SPELL_HEAL"]				= spell_heal,
	["SPELL_MISSED"]			= spell_missed,

	-- prefix: SPELL_PERIODIC_
	["SPELL_PERIODIC_DAMAGE"]	= spell_periodic_damage,
	["SPELL_PERIODIC_HEAL"]		= spell_periodic_heal,
	["SPELL_PERIODIC_MISSED"]	= spell_periodic_missed,

	-- prefix: DAMAGE_
	["DAMAGE_SHIELD"]			= damage_shield,
	["DAMAGE_SHIELD_MISSED"]	= damage_shield_missed,

	-- unknown
	["DAMAGE_SPLIT"]			= damage_split,
}

-----------------------------------------------------------------------------

-- COMBAT_LOG_* event handling; this is intended to replace UNIT_COMBAT event handling at some point, at least the "WOUND" event, so we can differentiate swing/"white" damage from spell damage
function AloftCombatText:OnCombatLogEvent(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, suffix1, suffix2, suffix3, suffix4, suffix5, suffix6, suffix7, suffix8, suffix9, suffix10, suffix11, suffix12)
	--[[
	ChatFrame7:AddMessage(("|cff%02x%02x%02x%s|r"):format(255, 255, 0, "AloftCombatText:OnCombatLogEvent(): ") .. tostring(timestamp) .. "/" .. tostring(subtype) .. "/" .. tostring(originGUId) .. "/" .. tostring(originName) .. "/" .. tostring(originFlags) .. "/" .. tostring(targetGUId) .. "/" .. tostring(targetName) .. "/" .. tostring(targetFlags)
		.. "/" .. tostring(suffix1) .. "/" .. tostring(suffix2) .. "/" .. tostring(suffix3) .. "/" .. tostring(suffix4) .. "/" .. tostring(suffix5) .. "/" .. tostring(suffix6) .. "/" .. tostring(suffix7) .. "/" .. tostring(suffix8) .. "/" .. tostring(suffix9) .. "/" .. tostring(suffix10) .. "/" .. tostring(suffix11) .. "/" .. tostring(suffix12))
	]]
	if targetGUId == playerTargetGUId then
		dispatchMethod = subtypeDispatchMethods[subtype]
		if dispatchMethod then
			dispatchMethod(timestamp, subtype, originGUId, originName, originFlags, targetGUId, targetName, targetFlags, suffix1, suffix2, suffix3, suffix4, suffix5, suffix6, suffix7, suffix8, suffix9, suffix10, suffix11, suffix12)
		end
	end
end

-----------------------------------------------------------------------------

function AloftCombatText:DumpOnUnitCombat(color, unit, event, flags, amount, typ)
	--[[
	ChatFrame7:AddMessage("AloftCombatText:DumpOnUnitCombat(): " .. tostring(unit) .. "/" .. tostring(event) .. "/" .. tostring(flags) .. "/" .. tostring(amount) .. "/" .. tostring(typ))
	ChatFrame7:AddMessage("AloftCombatText:DumpOnUnitCombat(): " .. tostring(color[1]) .. "-" .. tostring(color[2]) .. "-" .. tostring(color[3]) .. "-" .. tostring(profile.alpha)
		.. "/" .. floor(255*color[1]) .. "." .. floor(255*color[2]) .. "." .. floor(255*color[3]) .. "." .. floor(255*profile.alpha)
		.. "/" .. ("|c%02x%02x%02x%02xcombat text|r"):format(floor(255*profile.alpha), floor(255*color[1]), floor(255*color[2]), floor(255*color[3])))
	]]
end

-----------------------------------------------------------------------------

end)
