local Aloft = Aloft
if not Aloft then return end

-----------------------------------------------------------------------------

local AloftTargetTrackingData = Aloft:NewModule("TargetTrackingData", "AceEvent-2.0", "AceHook-2.1")

local roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------------

AloftTargetTrackingData.db = Aloft:AcquireDBNamespace("targetTracking")
Aloft:RegisterDefaults("targetTracking", "profile", {
	trackUnique	= false,
--[[
	focusEnable		= false,
	focusInterval	= 1.00, -- update group focus once a second (nominally)
]]
})

-----------------------------------------------------------------------------

local profile
local inInstance
local instanceType

AloftTargetTrackingData.nameplateIdToTargetId = { }
AloftTargetTrackingData.targetIdToNameplateId = { }
AloftTargetTrackingData.guidToUnitid = { }
AloftTargetTrackingData.unitidToGUId = { }

local UnitName				= UnitName
local UnitRace				= UnitRace
local UnitClass			= UnitClass
local UnitExists			= UnitExists
local UnitIsDead			= UnitIsDead
local UnitIsFriend			= UnitIsFriend
local UnitPlayerControlled	= UnitPlayerControlled
local GetRaidTargetIndex	= GetRaidTargetIndex
local IsInInstance			= IsInInstance

-----------------------------------------------------------------------------

function AloftTargetTrackingData:OnInitialize()
	profile = self.db.profile
end

function AloftTargetTrackingData:OnEnable()
	inInstance, instanceType = IsInInstance()
end

function AloftTargetTrackingData:OnDisable()
	self:UnregisterAllEvents()
end

-----------------------------------------------------------------------------

function AloftTargetTrackingData:ProvidesData()
end

function AloftTargetTrackingData:RequiresData()
	self:RegisterEvents()

	-- TODO: add race/class here?
	if Aloft:IsDataAvailable("unitid") then
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:RequiresData(): unitid+isTarget")
		return "unitid", "isTarget"
	else
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:RequiresData(): unitid")
		return "isTarget"
	end
end

function AloftTargetTrackingData:EnableDataSource()
end

function AloftTargetTrackingData:RegisterEvents()
	self:UnregisterAllEvents()

	-- ChatFrame7:AddMessage("AloftTargetTrackingData:RegisterEvents(): invoke")

	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA","OnPlayerEnteringWorld")
	-- self:RegisterEvent("UNIT_PET", "OnUnitPet")

	self:RegisterEvent("Aloft:OnIsTargetDataChanged", "OnIsTargetDataChanged")
	self:RegisterEvent("Aloft:OnIsFocusDataChanged", "OnIsFocusDataChanged")
	self:RegisterEvent("Aloft:OnUnitTargetDataChanged", "OnUnitTargetDataChanged")
	self:RegisterEvent("Aloft:OnUnitidDataChanged", "OnUnitidDataChanged")

	self:RegisterEvent("RAID_TARGET_UPDATE", "OnRaidTargetUpdate")
	self:RegisterEvent("RosterLib_RosterUpdated", "OnRosterUpdate")

	--[[
	if profile and profile.focusEnable then
		-- for update of focus targets
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:RegisterEvents(): schedule")
		self:ScheduleRepeatingEvent("AloftTargetTrackingData:CreateGroupFocusTracking", self.CreateGroupFocusTracking, profile.focusInterval, self)
	end
	]]
end

-----------------------------------------------------------------------------

function AloftTargetTrackingData:UpdateAll()
	self:ClearTracking()
	self:CreateTracking()
end

-----------------------------------------------------------------------------

function AloftTargetTrackingData:OnPlayerEnteringWorld()
	inInstance, instanceType = IsInInstance()
	if inInstance and instanceType == "arena" then
		local unitid
		for i = 1, 5 do
			unitid = "arena" .. i
			if UnitExists(unitid) then
				self:ClearUnitGUId(unitid)
				self:ClearUnitTracking(unitid, "")

				self:UpdateUnitGUId(unitid)
				self:UpdateUnitTracking(unitid, "")
			end
			unitid = "arenapet" .. i
			if UnitExists(unitid) then
				-- TODO: detect that an opponent has brought out a pet dynamically, and track that?
			end
		end
	else
		for i = 1, 5 do
			unitid = "arena" .. i
			if UnitExists(unitid) then
				self:ClearUnitTracking(unitid, "")
			end
			unitid = "arenapet" .. i
			if UnitExists(unitid) then
				-- TODO: detect that an opponent has brought out a pet dynamically, and clear that?
			end
		end
	end
	-- TODO: what to do if opponents are not loaded yet, and are returning "Unknown Entity"? detect via calls to UnitName() and re-loop?
end

--[[
function AloftTargetTrackingData:OnUnitPet(unitId)
	if unitId == "player" then
		self:ClearUnitGUId("pet")
		self:ClearUnitTracking("pet", "")
		if UnitExists("pet") then
			self:UpdateUnitGUId("pet")
			self:UpdateUnitTracking("pet", "")
		end
	end
end
]]

function AloftTargetTrackingData:OnRaidTargetUpdate()
	self:ClearTracking()
	self:CreateTracking()
end

-- rebuild the target tracking map
function AloftTargetTrackingData:OnRosterUpdate()
	self:ClearTracking()
	self:CreateTracking()
end

-- TODO: is this needed?
function AloftTargetTrackingData:OnIsTargetDataChanged(aloftData)
	self:TrackUnitTarget("player", aloftData)
end

function AloftTargetTrackingData:OnIsFocusDataChanged(aloftData)
	self:ClearUnitFocus("player")
	self:UpdateUnitFocus("player")
end

function AloftTargetTrackingData:OnUnitTargetDataChanged(aloftData, unitid)
	-- ChatFrame7:AddMessage("AloftTargetTrackingData:OnUnitTargetDataChanged(): " ..  tostring(unitid))
	if unitid then
		self:TrackUnitTarget(unitid, aloftData)
	end
end

-- TODO: do we really need to handle this event?
function AloftTargetTrackingData:OnUnitidDataChanged(aloftData)
	if aloftData.unitid then
		self:TrackUnitTarget(aloftData.unitid, aloftData)
	end
end

--[[
function AloftTargetTrackingData:CreateGroupFocusTracking()
	-- ChatFrame7:AddMessage("AloftTargetTrackingData:CreateGroupFocusTracking(): " ..  "enter")
	if roster then
		for unit in roster:IterateRoster(false) do -- no pets for focus
			-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateGroupFocus(): " ..  tostring(unit.unitid))
			self:ClearUnitFocus(unit.unitid)
			self:UpdateUnitFocus(unit.unitid)
		end
	end
end
]]

-----------------------------------------------------------------------------

-- clear the target tracking map
function AloftTargetTrackingData:ClearTracking()
	self:TriggerEvent("AloftTargetTrackingData:OnTargetTrackingDataClearAll")

	-- clean out targetid <> target nameplateid tracking
	for nameplateId in pairs(AloftTargetTrackingData.nameplateIdToTargetId) do
		for targetId in pairs(AloftTargetTrackingData.nameplateIdToTargetId[nameplateId]) do
			AloftTargetTrackingData.nameplateIdToTargetId[nameplateId][targetId] = nil
		end
		AloftTargetTrackingData.nameplateIdToTargetId[nameplateId] = nil
	end
	for targetId in pairs(AloftTargetTrackingData.targetIdToNameplateId) do
		AloftTargetTrackingData.targetIdToNameplateId[targetId] = nil
	end

	-- clean out GUID <> unitid tracking
	for guid in pairs(self.guidToUnitid) do
		self.guidToUnitid[guid] = nil
	end
	for unitid in pairs(self.unitidToGUId) do
		self.unitidToGUId[unitid] = nil
	end
end

-- build the target tracking map
function AloftTargetTrackingData:CreateTracking()
	-- update venue-based tracking
	self:OnPlayerEnteringWorld()

	-- update target tracking for the roster
	if roster then
		for unit in roster:IterateRoster(true) do
			-- ChatFrame7:AddMessage("AloftTargetTrackingData:CreateTracking(): " ..  tostring(unit.unitid))
			self:ClearUnitGUId(unit.unitid) -- clear the unitid's previous tracked GUID
			self:UpdateUnitGUId(unit.unitid) -- set the unitid's new tracked GUID

			self:TrackUnitTarget(unit.unitid, nil) -- track the unitid's target
			self:UpdateUnitTracking(unit.unitid, "") -- track the unitid itself (friendly/grouped)
		end
	end

	-- clear/update player GUId
	self:ClearUnitGUId("player")
	self:UpdateUnitGUId("player")

	--[[
	self:ClearUnitGUId("pet")
	self:ClearUnitTracking("pet", "")
	if UnitExists("pet") then
		self:UpdateUnitGUId("pet")
		self:UpdateUnitTracking("pet", "")
	end
	]]

	-- update player focus
	self:UpdateUnitFocus("player")
end

-----------------------------------------------------------------------------

-- adjust a unit's target tracking
function AloftTargetTrackingData:TrackUnitTarget(unitid, aloftData)
	if unitid then
		-- TODO: clean this up
		if unitid ~= "player" then
			self:ClearUnitTarget(unitid)
			self:UpdateUnitTarget(unitid)
		else
			-- player has no base unitid
			self:ClearUnitTarget("")
			self:UpdateUnitTarget("")
		end

		if not aloftData then
			if unitid ~= "player" then
				aloftData = Aloft:GetUnitNameplate(unitid)
			else
				aloftData = Aloft:GetTargetNameplate()
			end
		end	
	end

	-- note that in some cases, such as for player primary target changes, this will just filter OnIsTargetDataChanged events through as OnNameplateTrackingDataChanged
	if (aloftData) then
		self:TriggerEvent("AloftTargetTrackingData:OnNameplateTrackingDataChanged", aloftData)
	end
end

-----------------------------------------------------------------------------

-- get the unit's cached GUID
function AloftTargetTrackingData:GetUnitGUId(unitid)
	--[[
	for k, v in pairs(self.unitidToGUId) do
		ChatFrame7:AddMessage("AloftTargetTrackingData:GetUnitGUId(): content " .. tostring(k) .. "<>" .. tostring(v))
	end
	]]
	if unitid then
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:GetUnitGUId(): mapped " .. tostring(unitid) .. "<>" .. tostring(self.unitidToGUId[unitid]))
		return self.unitidToGUId[unitid]
	end
	return nil
end

-- get the unit's cached unitid
function AloftTargetTrackingData:GetGUIdUnitid(guid)
	--[[
	for k, v in pairs(self.guidToUnitid) do
		ChatFrame7:AddMessage("AloftTargetTrackingData:GetGUIdUnitid(): content " .. tostring(k) .. "<>" .. tostring(v))
	end
	]]
	if guid then
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:GetGUIdUnitid(): mapped " .. tostring(guid) .. "<>" .. tostring(self.guidToUnitid[guid]))
		return self.guidToUnitid[guid]
	end
	return nil
end

-- clear the unit's cached GUID
function AloftTargetTrackingData:ClearUnitGUId(unitid)
	if unitid then
		guid = self.unitidToGUId[unitid]
		if guid then
			self.guidToUnitid[guid] = nil
			-- ChatFrame7:AddMessage("AloftTargetTrackingData:ClearUnitGUId() " .. tostring(unitid) .. "><" .. tostring(guid))
		end
		self.unitidToGUId[unitid] = nil
	end
end

-- cache the unit's GUID
function AloftTargetTrackingData:UpdateUnitGUId(unitid)
	if unitid then
		guid = UnitGUID(unitid)
		if guid then
			self.guidToUnitid[guid] = unitid
			self.unitidToGUId[unitid] = guid
			-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitGUId() " .. tostring(self.guidToUnitid[guid]) .. "<>" .. tostring(self.unitidToGUId[unitid]))
		end
	end
end

-----------------------------------------------------------------------------

-- take the unitid target off of its former nameplate target (if any)
function AloftTargetTrackingData:ClearUnitTarget(unitid)
	self:ClearUnitTracking(unitid, "target")
end

-- put the unitid onto its current target (if any)
-- tracking potentially friendly units with raid icons is OK... we will not bother with updating target names unless the affected nameplate has no unitid/etc
function AloftTargetTrackingData:UpdateUnitTarget(unitid)
	self:UpdateUnitTracking(unitid, "target")
end

-----------------------------------------------------------------------------

-- take the unitid focus off of its former nameplate target (if any)
function AloftTargetTrackingData:ClearUnitFocus(unitid)
	if unitid == "player" then
		self:ClearUnitTracking("", "focus")
	end
end

-- put the unitid onto its current focus (if any)
-- tracking potentially friendly units with raid icons is OK... we will not bother with updating target names unless the affected nameplate has no unitid/etc
function AloftTargetTrackingData:UpdateUnitFocus(unitid)
	if unitid == "player" then
		self:UpdateUnitTracking("", "focus")
	end
end

-----------------------------------------------------------------------------

-- take the unitid off of its former nameplate target (if any)
function AloftTargetTrackingData:ClearUnitTracking(unitid, suffix)
	-- ChatFrame7:AddMessage("AloftTargetTrackingData:ClearUnitTracking(): " ..  "process " .. tostring(unitid) .. "/" .. tostring(suffix))

	if not unitid or not suffix then
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:ClearUnitTracking(): " ..  "no " .. tostring(unitid) .. "/" .. tostring(suffix))
		return
	end

	local targetId = unitid .. suffix
	local nameplateId = AloftTargetTrackingData.targetIdToNameplateId[targetId]
	self:TriggerEvent("AloftTargetTrackingData:OnTargetTrackingDataCleared", targetId, nameplateId)

	if (nameplateId) then
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:ClearUnitTracking(): " ..  "tostring(targetId) .. "/" .. tostring(suffix) .. "/"  .. tostring(targetId) .. "/"  ..  tostring(AloftTargetTrackingData.targetIdToNameplateId[targetId]) .. "/" .. tostring(AloftTargetTrackingData.nameplateIdToTargetId[nameplateId][targetId]))
		AloftTargetTrackingData.targetIdToNameplateId[targetId] = nil
		AloftTargetTrackingData.nameplateIdToTargetId[nameplateId][targetId] = nil
		for _ in pairs(AloftTargetTrackingData.nameplateIdToTargetId[nameplateId]) do return end -- stub out if there are still targetid associations
		AloftTargetTrackingData.nameplateIdToTargetId[nameplateId] = nil -- no further targetId associations, nil for garbage recovery
	end
end

-- put the unitid onto its current target (if any)
-- tracking potentially friendly units with raid icons is OK... we will not bother with updating target names unless the affected nameplate has no unitid/etc
function AloftTargetTrackingData:UpdateUnitTracking(unitid, suffix)
	-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitTracking(): " ..  "process " .. tostring(unitid) .. "/" .. tostring(suffix))

	if not unitid or not suffix then
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitTracking(): " ..  "no " .. tostring(unitid) .. "/" .. tostring(suffix))
		return
	end

	local targetId = unitid .. suffix
	-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitTracking(): targetId " .. tostring(targetId))

	if not UnitExists(targetId) then
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitTracking(): " ..  "not exists " .. tostring(targetId))
		return
	end
	if UnitIsDead(targetId) then
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitTracking(): " ..  "dead " .. tostring(targetId))
		return
	end
	--[[
	if UnitIsFriend("player", targetId) then
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitTracking(): " ..  "friend " .. tostring(targetId))
		return
	end
	]]
	--[[
	if UnitPlayerControlled(targetId) then
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitTracking(): " ..  "player controlled " .. tostring(targetId))
		return
	end
	]]
	-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitTracking(): " ..  "live/exists " .. tostring(targetId))

	local name = UnitName(targetId)
	local race, _ = UnitRace(targetId) -- assuming that if there is a race, the unit name is unique (i.e. a player char)
	local _, class = UnitClass(targetId) -- assuming that if there is a class, the unit name is unique (i.e. a player char)
	local raidTargetIndex = GetRaidTargetIndex(targetId) -- the target is required to have a raid icon assigned
	local isUnique = self:IsUnique(name, UnitClassification(targetId), UnitLevel(targetId))

	-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitTracking(): " .. tostring(targetId) .. "/" .. tostring(race) .. "/" .. tostring(class) .. "/" .. tostring(raidTargetIndex) .. "/" .. tostring(isUnique))
	if (race and class) or raidTargetIndex or isUnique then
		local nameplateId
		if (race and class) then
			nameplateId = self:FormatNameplateId(name, true, nil, nil)
		elseif raidTargetIndex or isUnique then
			nameplateId = self:FormatNameplateId(name, nil, raidTargetIndex, isUnique)
		end

		AloftTargetTrackingData.targetIdToNameplateId[targetId] = nameplateId
		if (not AloftTargetTrackingData.nameplateIdToTargetId[nameplateId]) then AloftTargetTrackingData.nameplateIdToTargetId[nameplateId] = { } end
		AloftTargetTrackingData.nameplateIdToTargetId[nameplateId][targetId] = true
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitTracking(): trigger " .. tostring(unitid) .. "/" .. tostring(suffix) .. "/"  .. tostring(targetId) .. "/"  ..  tostring(AloftTargetTrackingData.targetIdToNameplateId[targetId]) .. "/" .. tostring(AloftTargetTrackingData.nameplateIdToTargetId[nameplateId][targetId]))

		-- ChatFrame7:AddMessage("AloftTargetTrackingData:UpdateUnitTracking(): " .. tostring(unitid) .. "/" .. tostring(suffix) .. "/"  .. tostring(targetId) .. "/"  ..  tostring(AloftTargetTrackingData.targetIdToNameplateId[targetId]) .. "/" .. tostring(AloftTargetTrackingData.nameplateIdToTargetId[nameplateId][targetId]))
		self:TriggerEvent("AloftTargetTrackingData:OnTargetTrackingDataUpdated", targetId, nameplateId)
	end
end

-----------------------------------------------------------------------------

function AloftTargetTrackingData:IsTargetIdToNameplateIdTracking()
	for k in pairs(AloftTargetTrackingData.targetIdToNameplateId) do
		-- ChatFrame7:AddMessage("AloftTargetTrackingData:IsTargetIdToNameplateIdTracking(): TGTGT " .. tostring(k) .. " has target " .. tostring(AloftTargetTrackingData.targetIdToNameplateId[k]))
		if k then return true end -- there is tracking
	end
	return false -- there is no tracking
end

-- construct a nameplate id from a nameplate
function AloftTargetTrackingData:GetNameplateId(aloftData)
	if ((aloftData.type == "hostilePlayer" or aloftData.type == "hostile") and ((aloftData.class and aloftData.race) or (Aloft.showClassColorInVKey and Aloft:GetClassByColor(aloftData.originalHealthBarR, aloftData.originalHealthBarG, aloftData.originalHealthBarB)))) or aloftData.type == "friendlyPlayer" then
		--[[
		ChatFrame7:AddMessage("AloftTargetTrackingData:GetNameplateId(): " .. tostring(aloftData.name) .. "/" .. tostring(aloftData.type) .. "/" .. tostring(aloftData.race) .. "/" .. tostring(aloftData.class))
		local color = Aloft:GetClassByColor(aloftData.originalHealthBarR, aloftData.originalHealthBarG, aloftData.originalHealthBarB)
		if color then
			ChatFrame7:AddMessage("AloftTargetTrackingData:GetNameplateId(): " .. tostring(color[1]) .. "-" .. tostring(color[2]) .. "-" .. tostring(color[3])
			.. "/" .. floor(255*color[1]) .. "." .. floor(255*color[2]) .. "." .. floor(255*color[3])
			.. "/" .. ("|cff%02x%02x%02xcolor|r"):format(floor(255*color[1]), floor(255*color[2]), floor(255*color[3])))
		end
		]]

		-- this is a player
		return self:FormatNameplateId(aloftData.name, true, nil, nil)
	else
		local raidIconRegion = aloftData.raidIconRegion
		local raidIconIndex = (raidIconRegion:IsShown() and self:GetRaidIconIndex(raidIconRegion)) or nil
		local isUnique = self:IsUnique(aloftData.name, aloftData.classification, aloftData.level)

		-- ChatFrame7:AddMessage("AloftTargetTrackingData:GetNameplateId(): " .. tostring(aloftData.name) .. "/" .. tostring(aloftData.type) .. "/" .. tostring(raidIconIndex) .. "/" .. tostring(isUnique))

		-- this is uniquely identifiable
		return self:FormatNameplateId(aloftData.name, nil, raidIconIndex, isUnique)
	end

	-- none of the above
	return nil
	-- TODO: fix this if/else logic, this determination is not made here anyway
end

--[[
-- construct a nameplate id from a nameplate
function AloftTargetTrackingData:GetNameplateId(aloftData)
	-- TODO: add boss icon?
	-- TODO: can we detect the combination of rare and elite?
	local race = aloftData.race -- assuming that if there is a race, the unit name is unique (i.e. a player char)
	local class = aloftData.class -- assuming that if there is a class, the unit name is unique (i.e. a player char)
	local raidIconRegion = aloftData.raidIconRegion
	local raidIconIndex = (raidIconRegion:IsShown() and self:GetRaidIconIndex(raidIconRegion)) or nil
	-- ChatFrame7:AddMessage("AloftTargetTrackingData:GetNameplateId(): " .. tostring(aloftData.name) .. "/" .. tostring(race) .. "/" .. tostring(class) .. "/" .. tostring(raidIconIndex))

	return self:FormatNameplateId(aloftData.name, raidIconIndex, race, class)
end
]]

-- construct a nameplate id from a set of parameters
function AloftTargetTrackingData:FormatNameplateId(name, player, raidIconIndex, unique)
	if player then
		return name
	elseif raidIconIndex or unique then -- must be uniquely identifiable
		return name .. ((raidIconIndex and ("-" .. tostring(raidIconIndex))) or "") .. ((unique and ("-Unique")) or "")
	end
	return nil
end

--[[
-- construct a nameplate id from a set of parameters
function AloftTargetTrackingData:FormatNameplateId(name, raidIconIndex, race, class)
	if (race and class) or raidIconIndex then
		local nameplateId = name ..
			((raidIconIndex and ("-" .. tostring(raidIconIndex))) or "") ..
			(((race and class) and ("-" .. tostring(race) .. "-" .. tostring(class))) or "")
		return nameplateId
	end
	return nil
end
]]

-- NOTE: this is pretty kludgy; where these icons appear in Blizzard's texture file is an internal detail, and could change w/o warning
function AloftTargetTrackingData:GetRaidIconIndex(raidIconRegion)
	local left, right, top, bottom = raidIconRegion:GetTexCoord()
	index = ((right / 0.25) * 4) + (left / 0.25) + 1 -- offsets within the Blizzard texture file, empirically reverse engineered via Texture:GetTexCoord()
	return index
end

function AloftTargetTrackingData:IsUnique(name, classification, level)
	-- ChatFrame7:AddMessage("AloftTargetTrackingData:IsUnique(): " .. tostring(name) .. "/" .. tostring(classification) .. "/" .. tostring(level) .. "|" .. tostring(profile.trackUniqueElite) .. "/" .. tostring(inInstance) .. "/" .. tostring(((profile.trackUniqueElite and ((inInstance and classification == "elite" and (not level or level < 0)) or classification == "rareelite" or classification == "rare" or classification == "worldboss") and name ~= "[invalid]") and true) or nil))
	return ((profile.trackUniqueElite and ((inInstance and classification == "elite" and (not level or level < 0)) or classification == "rareelite" or classification == "rare" or classification == "worldboss") and name ~= "[invalid]") and true) or nil
end

--[[ raid icon offsets as of WoW 3.0.2, 2008/10/22
1/0.00/0.00/0.00/0.25
2/0.25/0.00/0.25/0.25
3/0.50/0.00/0.50/0.25
4/0.75/0.00/0.75/0.25
5/0.00/0.25/0.00/0.50
6/0.25/0.25/0.25/0.50
7/0.50/0.25/0.50/0.50
8/0.75/0.25/0.75/0.50
]]

-----------------------------------------------------------------------------
