-- Runevolution

-- globals

Runevolution_version = "v1.1.0"

-- saved variables, per character
RunevolutionData = {
    loc = { x=0 , y=-100 },
    runesize = 24,
    cooldowndistance = 5,
    show_ooc = true,
    show_rp = true,
    show_back = true,
    fix_rp = false,
    show_diseases = false,
    diseases_up = false,
    hide_bliz = false,
    show_all_diseases = false,
    static = false,
    REversion = Runevolution_version   -- version mark for pref updating
}

RunevolutionRuneTextures = {
    ["blood"] = "Interface\\Addons\\Runevolution\\Blood.blp",
    ["bloodoff"] = "Interface\\Addons\\Runevolution\\Blood-Off.blp",
    ["bloodback"] = "Interface\\Addons\\Runevolution\\Blood-back.blp",
    ["unholy"] = "Interface\\Addons\\Runevolution\\Unholy.blp",
    ["unholyoff"] = "Interface\\Addons\\Runevolution\\Unholy-Off.blp",
    ["unholyback"] = "Interface\\Addons\\Runevolution\\Unholy-back.blp",
    ["frost"] = "Interface\\Addons\\Runevolution\\Frost.blp",
    ["frostoff"] = "Interface\\Addons\\Runevolution\\Frost-Off.blp",
    ["frostback"] = "Interface\\Addons\\Runevolution\\Frost-back.blp",
    ["death"] = "Interface\\Addons\\Runevolution\\Death.blp",
    ["deathoff"] = "Interface\\Addons\\Runevolution\\Death-Off.blp",
    ["deathback"] = "Interface\\Addons\\Runevolution\\Death-back.blp",
}

RunevolutionDiseaseList = {
    ["Blood Plague"] = { unit="target" , type="debuff" },
    ["Frost Fever"] = { unit="target" , type="debuff" },
    ["Unholy Blight"] = { unit="player" , type="buff" },
    --["Ebon Plague"] = { unit="target" , type="debuff" , icon="Spell_Shadow_NetherCloak" }
}

RunevolutionFrames = {}
RunevolutionTextures = {}

-- relative positions of runes at start
RunevolutionXOffset = {
    [1] = -6,
    [2] = -5,
    [3] = -1,
    [4] = -2,
    [5] = -4,
    [6] = -3
}
RunevolutionYOffset = {
    [1] = 0,
    [2] = 0,
    [3] = 0,
    [4] = 0,
    [5] = 0,
    [6] = 0
}
RunevolutionTimeReady = {
    [1] = 1,
    [2] = 2,
    [3] = 3,
    [4] = 4,
    [5] = 5,
    [6] = 6
}

-- v2
-- coordinates allow for arbitrary rune paths
-- each pair of x,y coords corresponds to an x-offset
-- e.g. x-offset = -1 takes the [-1] pair of coordinates
-- when a rune is between x offsets, position is linear interpolation of neighboring positions
RunevolutionCoordinates = {
    ["standard"] = {
        -- ready rune positions
        [-6] = {-6,0},
        [-5] = {-5,0},
        [-4] = {-4,0},
        [-3] = {-3,0},
        [-2] = {-2,0},
        [-1] = {-1,0},
        -- cooling rune positions
        [0] = {0,0},
        [1] = {1,0},
        [2] = {2,0},
        [3] = {3,0},
        [4] = {4,0},
        [5] = {5,0},
        [6] = {6,0},
        [7] = {7,0},
        [8] = {8,0},
        [9] = {9,0},
        [10] = {10,0},
        ["newready"] = -1, -- absolute position of newly ready rune
        ["shiftready"] = -1 -- shift ready runes to left
    },
    ["left justify"] = {
        -- ready rune positions
        [-6] = {-1,0},
        [-5] = {-2,0},
        [-4] = {-3,0},
        [-3] = {-4,0},
        [-2] = {-5,0},
        [-1] = {-6,0},
        -- cooling rune positions
        [0] = {0,0},
        [1] = {1,0},
        [2] = {2,0},
        [3] = {3,0},
        [4] = {4,0},
        [5] = {5,0},
        [6] = {6,0},
        [7] = {7,0},
        [8] = {8,0},
        [9] = {9,0},
        [10] = {10,0},
        ["newready"] = "right", -- position freshly ready runes at first empty right hand position
        ["shiftready"] = 0 -- no shift for runes already off cooldown
    },
    ["loop1"] = {
        -- ready rune positions
        [-6] = {-6,0},
        [-5] = {-5,0},
        [-4] = {-4,0},
        [-3] = {-3,0},
        [-2] = {-2,0},
        [-1] = {-1,0},
        -- cooling rune positions
        [0] = {-1,-1},
        [10] = {-6,-1},
        ["newready"] = -1, -- absolute position of newly ready rune
        ["shiftready"] = -1 -- shift ready runes to left
    }
}
RunevolutionCoordSet = "standard"

for k,v in pairs( RunevolutionDiseaseList ) do
    RunevolutionXOffset[k] = 0
    RunevolutionYOffset[k] = -1
end

RunevolutionPulseDuration = 1/2 -- time in seconds it takes for pulse animation to finish
RunevolutionPulseFactor = 1.5 -- initial scale multiplier of pulse
RunevolutionPulseList = { -- animation times on pulse
    [1] = 0,
    [2] = 0,
    [3] = 0,
    [4] = 0,
    [5] = 0,
    [6] = 0,
    ['rp'] = 0,
    ['Blood Plague'] = 0,
    ['Frost Fever'] = 0
}
RunevolutionFlashList = {} -- pulsing alpha, to indicate diseases near expiring

RunevolutionTimeReady = {}

RunevolutionHBtimer = 0 -- kluge for timing Howling Blast, until they fix it

-- Initialize

function Runevolution_OnLoad()

    local _,engClass = UnitClass("player")
    if engClass ~= "DEATHKNIGHT" then
        return
    end

    -- register events
    
    --- initialization events
    this:RegisterEvent("PLAYER_ENTERING_WORLD")
    this:RegisterEvent("VARIABLES_LOADED")
    
    -- rune events
    --this:RegisterEvent("RUNE_POWER_UPDATE")
    --this:RegisterEvent("RUNE_TYPE_UPDATE")
    
    -- kluge Howling Blast event, so we can time cooldown until they fix the tooltip
    this:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	
	-- options init
	SlashCmdList["RUNEV"] = Runevolution_Options
	SLASH_RUNEV1 = "/runev"

end

-- slash handler
function Runevolution_Options(msg)
    
    local param;
    msg,param = strsplit(" ",msg,2);
    
    if msg == "x" then
        RunevolutionData.loc["x"] = tonumber(param)
        Runevolution_UpdateRunePositions()
    elseif msg == "y" then
        RunevolutionData.loc["y"] = tonumber(param)
        Runevolution_UpdateRunePositions()
    elseif msg == "cooldown" then
        RunevolutionData.cooldowndistance = tonumber(param)
        Runevolution_UpdateRunePositions()
    elseif msg == "size" then
        RunevolutionData.runesize = tonumber(param)
        for i = 1,6 do
            RunevolutionFrames[i]:SetWidth( RunevolutionData.runesize )
            RunevolutionFrames[i]:SetHeight( RunevolutionData.runesize )
        end
        Runevolution_UpdateRunePositions()
    elseif msg == "reset" then
        Runevolution_AttemptReset()
        Runevolution_UpdateRuneTypes()
        Runevolution_UpdateRunePositions()
    elseif msg == "order" then
        local order = { strsplit( "," , param ) }
        for i = 1,6 do
            local _,_,ready = GetRuneCooldown( i )
            if ready then
                RunevolutionXOffset[ i ] = -7 + tonumber( order[ i ] )
            end
        end
        Runevolution_UpdateRuneTypes()
        Runevolution_UpdateRunePositions()
    elseif msg == "showooc" then
        RunevolutionData.show_ooc = not RunevolutionData.show_ooc
    elseif msg == "showrp" then
        RunevolutionData.show_rp = not RunevolutionData.show_rp
    elseif msg == "showback" then
        RunevolutionData.show_back = not RunevolutionData.show_back
        for i = 7,12 do
            if RunevolutionData.show_back then
                RunevolutionFrames[i]:Show()
            else
                RunevolutionFrames[i]:Hide()
            end
        end
    elseif msg == "fixrp" then
        RunevolutionData.fix_rp = not RunevolutionData.fix_rp
        Runevolution_UpdateRunePositions()
    elseif msg == "diseases" then
        RunevolutionData.show_diseases = not RunevolutionData.show_diseases
    elseif msg == "diseasepos" then
        RunevolutionData.diseases_up = not RunevolutionData.diseases_up
    elseif msg == "hidebliz" then
        RunevolutionData.hide_bliz = not RunevolutionData.hide_bliz
        if RunevolutionData.hide_bliz then
            RuneFrame:Hide()
        else
            RuneFrame:Show()
        end
    elseif msg == "alldiseases" then
        RunevolutionData.show_all_diseases = not RunevolutionData.show_all_diseases
    elseif msg == "static" then
        RunevolutionData.static = not RunevolutionData.static
    else
        -- print help
        reprint("Runevolution "..Runevolution_version.." commands: (current values)")
        reprint("    /runev x # : adjust x position ("..RunevolutionData.loc["x"]..")" )
        reprint("    /runev y # : adjust y position ("..RunevolutionData.loc["y"]..")" )
        reprint("    /runev cooldown # : cooldown length in rune widths ("..RunevolutionData.cooldowndistance..")" )
        reprint("    /runev size # : pixel size of runes ("..RunevolutionData.runesize..")" )
        reprint("    /runev showooc : toggle out-of-combat visibility of runes ("..tostring(RunevolutionData.show_ooc)..")" )
        reprint("    /runev showrp : toggle visibility of runic power ("..tostring(RunevolutionData.show_rp)..")" )
        reprint("    /runev showback : toggle rune backgrounds ("..tostring(RunevolutionData.show_back)..")" )
        reprint("    /runev fixrp : toggle fixed/floating runic power ("..tostring(RunevolutionData.fix_rp)..")" )
        reprint("    /runev diseases : toggle showing diseases on target ("..tostring(RunevolutionData.show_diseases)..")" )
        reprint("    /runev diseasepos : toggle position of diseases, up or down ("..tostring(RunevolutionData.diseases_up)..")" )
        reprint("    /runev alldiseases : show diseases cast by others ("..tostring(RunevolutionData.show_all_diseases)..")" )
        reprint("    /runev static : toggle static mode ("..tostring(RunevolutionData.static)..")" )
        reprint("    /runev reset : reset the order of runes")
        reprint("    /runev order #,#,#,#,#,# : set order of runes; 1=far left, 6=far right")
        reprint("    /runev hidebliz : hide the default built-in rune frame")
    end

end

function reprint(x)
    if x then DEFAULT_CHAT_FRAME:AddMessage("|cffffff00"..x) end
end

function Runevolution_RUNE_POWER_UPDATE( runenum , ready )
    --reprint( event.." : "..tostring(params[1]).." / "..tostring(params[2]) )
    
    --local params = { select(1,...) }
    --local runenum = params[1]
    --local ready = params[2]
    
    if runenum < 7 then
        if ready then
            -- a real rune that just became ready
            
            if not RunevolutionData.static then
                -- drift mode
                
                if RunevolutionCoordinates[ RunevolutionCoordSet ][ "shiftready" ] == -1 then
                    -- move all currently ready runes LEFT one place
                    for i = 1,6 do
                        if RunevolutionXOffset[i] < 0 then
                            RunevolutionXOffset[i] = RunevolutionXOffset[i] - 1
                        end
                    end
                elseif RunevolutionCoordinates[ RunevolutionCoordSet ][ "shiftready" ] == 1 then
                    -- move all currently ready runes RIGHT one place
                    for i = 1,6 do
                        if RunevolutionXOffset[i] < 0 then
                            RunevolutionXOffset[i] = RunevolutionXOffset[i] + 1
                        end
                    end
                end
                
                if type(RunevolutionCoordinates[ RunevolutionCoordSet ][ "newready" ]) == "number" then
                    -- absolute position
                    RunevolutionXOffset[ runenum ] = RunevolutionCoordinates[ RunevolutionCoordSet ][ "newready" ]
                    RunevolutionYOffset[ runenum ] = 0 -- be sure to reset any vertical stagger
                elseif RunevolutionCoordinates[ RunevolutionCoordSet ][ "newready" ] == "right" then
                    -- find first open rune spot and place it there
                    local firstopen = min( RunevolutionXOffset[1] , RunevolutionXOffset[2] , RunevolutionXOffset[3] , RunevolutionXOffset[4] , RunevolutionXOffset[5] , RunevolutionXOffset[6] ) - 1
                    if firstopen < -6 then firstopen = -6 end
                    RunevolutionXOffset[ runenum ] = firstopen
                    RunevolutionYOffset[ runenum ] = 0 -- be sure to reset any vertical stagger
                end
                
            else
                -- static mode, leave in place
            end
            RunevolutionPulseList[ runenum ] = RunevolutionPulseDuration -- pulse it
        else
            -- a rune just got used
            local t = RunevolutionXOffset[ runenum ]
            -- position rune to far right
            
            if not RunevolutionData.static then
                RunevolutionXOffset[ runenum ] = RunevolutionData.cooldowndistance -- start cooldown
            else
                -- static mode
                -- move used rune to right end of six runes
                -- shift others left one place, if were to right of rune we just used
                for i = 1,6 do
                    if RunevolutionXOffset[i] > RunevolutionXOffset[ runenum ] then
                        RunevolutionXOffset[i] = RunevolutionXOffset[i] - 1
                    end
                end
                RunevolutionXOffset[ runenum ] = -1 -- far right position
            end
            
            -- check for another rune near that same cooldown position
            -- if so, stagger the runes vertically
            for i = 1,6 do
                if i ~= runenum and abs(RunevolutionXOffset[ i ] - RunevolutionXOffset[ runenum ]) < 1/2 then
                    RunevolutionYOffset[ runenum ] = -0.3
                    RunevolutionYOffset[ i ] = 0.3
                end
            end
            
            if not RunevolutionData.static then
                -- move all ready runes that were left of this one to the right one place
                for i = 1,6 do
                    if RunevolutionXOffset[i] < t then
                        RunevolutionXOffset[i] = RunevolutionXOffset[i] + 1
                    end
                end
            end
            
        end
        
        Runevolution_UpdateRuneTypes()
        Runevolution_UpdateRunePositions()
        
    end
end

-- event handler
RunevolutionRuneUpdateQueue = {}
function Runevolution_OnEvent(self,event,...)

    local params = { select(1,...) }

    if event == "RUNE_POWER_UPDATE" then
        
        table.insert( RunevolutionRuneUpdateQueue , {params[1],params[2]} )
        --Runevolution_RUNE_POWER_UPDATE(...)
        
    elseif event=="RUNE_TYPE_UPDATE" then
    
        Runevolution_UpdateRuneTypes()
    
    elseif event == "PLAYER_ENTERING_WORLD" then

        -- nothing here yet
        --reprint("PLAYER_ENTERING_WORLD")
        
        Runevolution_UpdateRuneTypes()
        
        -- confirm that disease icons are loaded
        for k,v in pairs( RunevolutionDiseaseList ) do
            local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange 
= GetSpellInfo( k )
            if v.icon then
                icon = "Interface\\Icons\\"..v.icon
            end
            RunevolutionTextures[k]:SetTexture( icon )
            RunevolutionPulseList[ k ] = 0
            RunevolutionXOffset[ k ] = 0 -- init cooldown/disease positions
        end
        
        if RunevolutionData.hide_bliz then
            RuneFrame:Hide()
        else
            RuneFrame:Show()
        end
        
        Runevolution_AttemptReset() -- fix for mussed runes after death
        
    elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
    
        -- kluge for Howling Blast timing
        local unitID,spellName,spellRank = select(1,...)
    	if ( unitID == "player" and spellName == "Howling Blast" ) then
            -- start cooldown timer
            RunevolutionHBtimer = 5 -- 5 seconds until it's ready again
    	end
        
    elseif event == "VARIABLES_LOADED" then
                
        -- initialize
        --reprint("VARIABLES_LOADED")
        
        if RunevolutionData.REversion == "v1.0.0" then
            -- add new prefs for v1.0.1
            RunevolutionData.show_ooc = true
            RunevolutionData.REversion = "v1.0.1"
        end
        
        if RunevolutionData.REversion == "v1.0.1" then
            -- add new prefs for v1.0.2
            RunevolutionData.show_rp = true
            RunevolutionData.REversion = "v1.0.2"
        end
        
        if RunevolutionData.REversion == "v1.0.2" or RunevolutionData.REversion == "v1.0.3" then
            -- add new prefs for v1.0.4
            RunevolutionData.show_back = true
            RunevolutionData.fix_rp = false
            RunevolutionData.REversion = "v1.0.4"
        end
        
        if RunevolutionData.REversion == "v1.0.4" then
            -- add new prefs for v1.0.5
            RunevolutionData.show_diseases = false
            RunevolutionData.REversion = "v1.0.5"
        end
        
        if RunevolutionData.REversion == "v1.0.5" then
            -- add new prefs for v1.0.6
            RunevolutionData.diseases_up = false
            RunevolutionData.REversion = "v1.0.6"
        end
        
        if RunevolutionData.REversion == "v1.0.6" then
            -- add new prefs for v1.0.7
            RunevolutionData.hide_bliz = false
            RunevolutionData.show_all_diseases = false
            RunevolutionData.REversion = "v1.0.7"
        end
        
        if RunevolutionData.REversion == "v1.0.7" then
            -- add new prefs for v1.0.8
            RunevolutionData.static = false
            RunevolutionData.REversion = "v1.0.8"
        end
        
        Runevolution_CreateFrames()
        
    end

end


-- make the frames from stored settings
RunevolutionFrameMark = nil
RunevolutionFrameMarkTexture = nil
RunevolutionParentContainer = nil
RunevolutionFrameRP = nil
RunevolutionFrameRPText = nil
function Runevolution_CreateFrames()

    -- get object for parent frame
    RunevolutionParentContainer = CreateFrame("Frame" , "RunevolutionFrameContainer" , UIParent )
    RunevolutionParentContainer:SetFrameStrata("LOW")
    RunevolutionParentContainer:SetWidth( 100 )
    RunevolutionParentContainer:SetHeight( 100 )
    RunevolutionParentContainer:SetPoint( "CENTER" , 0 , 0 )
    RunevolutionParentContainer:Show()

    -- make a frame and texture for each rune
    for i = 1,6 do
        -- parent frame
        RunevolutionFrames[i] = CreateFrame("Frame" , "RunevolutionFrame"..i , RunevolutionParentContainer )
        RunevolutionFrames[i]:SetFrameStrata("LOW")
        -- texture
        RunevolutionTextures[i] = RunevolutionFrames[i]:CreateTexture(nil,"BACKGROUND")
        --RunevolutionTextures[i]:SetBlendMode( "ADD" )
        RunevolutionTextures[i]:SetBlendMode( "BLEND" )
        RunevolutionTextures[i]:SetAllPoints( RunevolutionFrames[i] )
        
        RunevolutionFrames[i]:SetWidth( RunevolutionData.runesize )
        RunevolutionFrames[i]:SetHeight( RunevolutionData.runesize )
        
        RunevolutionFrames[i]:SetPoint( "CENTER" , RunevolutionData.loc["x"] + RunevolutionXOffset[i]*RunevolutionData.runesize , RunevolutionData.loc["y"] )
        RunevolutionFrames[i]:Show()
        
        -- text
        RunevolutionFrames[i].text = RunevolutionFrames[i]:CreateFontString( "RunevolutionFrameRuneString"..i )
        RunevolutionFrames[i].text:SetFont("Fonts\\FRIZQT__.TTF", RunevolutionData.runesize/1.5 , "OUTLINE")
        RunevolutionFrames[i].text:SetPoint("CENTER" , RunevolutionFrames[i] , "CENTER" , 0 , 0)
        RunevolutionFrames[i].text:SetText( "|cFFFFFFFF10" )
        RunevolutionFrames[i].text:Hide() -- hide until rune is on cooldown
    end
    
    -- make background shadows
    for i = 7,12 do
        -- parent frame
        RunevolutionFrames[i] = CreateFrame("Frame" , "RunevolutionFrame"..i , RunevolutionFrames[i-6] )
        RunevolutionFrames[i]:SetFrameStrata( "BACKGROUND" )
        -- texture
        RunevolutionTextures[i] = RunevolutionFrames[i]:CreateTexture(nil,"BACKGROUND")
        RunevolutionTextures[i]:SetBlendMode( "BLEND" )
        RunevolutionTextures[i]:SetAllPoints( RunevolutionFrames[i] )
        --RunevolutionTextures[i]:SetVertexColor( 1 , 0 , 0 )
        
        RunevolutionFrames[i]:SetWidth( RunevolutionData.runesize )
        RunevolutionFrames[i]:SetHeight( RunevolutionData.runesize )
        
        RunevolutionFrames[i]:SetPoint( "CENTER" , RunevolutionFrames[i-6] , "CENTER" , 0 , 0 )
        RunevolutionFrames[i]:Show()
        if not RunevolutionData.show_back then
            RunevolutionFrames[i]:Hide()
        end
    end
    
    -- make disease trackers
    for k,v in pairs( RunevolutionDiseaseList ) do
        Runevolution_CreateCooldownIcon( k )
    end
    
    -- make end-of-cooldown marker
    RunevolutionFrameMark = CreateFrame("Frame","RunevolutionFrameMark" , RunevolutionParentContainer )
    RunevolutionFrameMark:SetFrameStrata("LOW")
    RunevolutionFrameMarkTexture = RunevolutionFrameMark:CreateTexture(nil,"BACKGROUND")
    RunevolutionFrameMarkTexture:SetBlendMode( "BLEND" )
    RunevolutionFrameMarkTexture:SetAllPoints( RunevolutionFrameMark )
    RunevolutionFrameMark:SetWidth( 4 )
    RunevolutionFrameMark:SetHeight( RunevolutionData.runesize )
    RunevolutionFrameMark:SetPoint( "CENTER" , RunevolutionData.loc["x"] - RunevolutionData.runesize/2 , RunevolutionData.loc["y"] )
    RunevolutionFrameMarkTexture:SetTexture( 1 , 1 , 1 , .75 )
    RunevolutionFrameMark:Hide()
    
    -- make runic power indicator
    RunevolutionFrameRP = CreateFrame("Frame","RunevolutionFrameRP" , RunevolutionParentContainer )
    RunevolutionFrameRP:SetFrameStrata("LOW")
    RunevolutionFrameRP:SetWidth( 4 )
    RunevolutionFrameRP:SetHeight( 4 )
    RunevolutionFrameRP:SetPoint( "CENTER" , RunevolutionData.loc["x"] , RunevolutionData.loc["y"] )
    RunevolutionFrameRP:Show()
    
    RunevolutionFrameRPText = RunevolutionFrameRP:CreateFontString( "RunevolutionFrameRPString" )
	--RunevolutionFrameRPText:SetFontObject("GameFontNormal")
	RunevolutionFrameRPText:SetFont("Fonts\\FRIZQT__.TTF", RunevolutionData.runesize/1.5 , "OUTLINE")
	RunevolutionFrameRPText:SetPoint("CENTER" , RunevolutionFrameRP , "CENTER" , -7*RunevolutionData.runesize , 0)
	RunevolutionFrameRPText:SetText( "|cFF00B8F5100" )
	
	-- kluge for timing Howling Blast
	RunevolutionXOffset[ "Howling Blast" ] = 1
	RunevolutionYOffset[ "Howling Blast" ] = 1
	Runevolution_CreateCooldownIcon( "Howling Blast" )
	RunevolutionFrames["Howling Blast"]:ClearAllPoints()
	RunevolutionFrames["Howling Blast"]:SetPoint( "CENTER" , RunevolutionData.loc["x"] + RunevolutionXOffset["Howling Blast"]*RunevolutionData.runesize , RunevolutionData.loc["y"] + RunevolutionYOffset["Howling Blast"]*RunevolutionData.runesize )
	RunevolutionFrames["Howling Blast"].text:ClearAllPoints()
	RunevolutionFrames["Howling Blast"].text:SetPoint("BOTTOM" , RunevolutionFrames["Howling Blast"] , "TOP" , 0 , 0)
    
    -- set initial textures
    Runevolution_UpdateRuneTypes()
    Runevolution_UpdateRunePositions()

end

function Runevolution_CreateCooldownIcon( cooldownname )

    local iconsizefactor = 0.8 -- shrink the blizz icons a bit
    
    RunevolutionFrames[cooldownname] = CreateFrame("Frame" , "RunevolutionFrameCooldown"..cooldownname , RunevolutionParentContainer )
    RunevolutionFrames[cooldownname]:SetFrameStrata( "BACKGROUND" )
    RunevolutionTextures[cooldownname] = RunevolutionFrames[cooldownname]:CreateTexture(nil,"BACKGROUND")
    RunevolutionTextures[cooldownname]:SetBlendMode( "BLEND" )
    RunevolutionTextures[cooldownname]:SetAllPoints( RunevolutionFrames[cooldownname] )
    local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange 
= GetSpellInfo( cooldownname )
    RunevolutionTextures[cooldownname]:SetTexture( icon )
    RunevolutionFrames[cooldownname]:SetWidth( RunevolutionData.runesize * iconsizefactor )
    RunevolutionFrames[cooldownname]:SetHeight( RunevolutionData.runesize * iconsizefactor )
    RunevolutionFrames[cooldownname]:SetPoint( "CENTER" , RunevolutionData.loc["x"] + RunevolutionXOffset[cooldownname]*RunevolutionData.runesize , RunevolutionData.loc["y"] )
    RunevolutionFrames[cooldownname]:Hide()
    
    RunevolutionFrames[cooldownname].text = RunevolutionFrames[cooldownname]:CreateFontString( "RunevolutionFrameString"..cooldownname )
	--RunevolutionFrameRPText:SetFontObject("GameFontNormal")
	RunevolutionFrames[cooldownname].text:SetFont("Fonts\\FRIZQT__.TTF", RunevolutionData.runesize/2 , "OUTLINE")
	RunevolutionFrames[cooldownname].text:SetPoint("TOP" , RunevolutionFrames[cooldownname] , "BOTTOM" , 0 , 0)
	RunevolutionFrames[cooldownname].text:SetText( "|cFFFFFFFF11s" )
    
end


function Runevolution_UpdateRuneTypes()
    -- sets rune types
    for i = 1,6 do
        local rtype = GetRuneType( i )
        local _,_,ready = GetRuneCooldown( i ) 
        local texture = nil
        local textname = ""
        if rtype == 1 then
            textname = "blood"
        elseif rtype == 2 then
            textname = "unholy"
        elseif rtype == 3 then
            textname = "frost"
        elseif rtype == 4 then
            textname = "death"
        end
        local backtexture = textname.."back"
        if not ready then
            textname = textname.."off"
        end
        texture = RunevolutionRuneTextures[ textname ]
        RunevolutionTextures[i]:SetTexture( texture )
        RunevolutionTextures[i+6]:SetTexture( RunevolutionRuneTextures[ backtexture ] )
    end
end


function Runevolution_UpdateRunePositions()
    
    local x,y
    for i = 1,6 do
        if RunevolutionCoordinates[RunevolutionCoordSet][ RunevolutionXOffset[i] ] then
            -- exact x offset in set, so retrieve coordinates
            x = RunevolutionCoordinates[RunevolutionCoordSet][ RunevolutionXOffset[i] ][1]
            y = RunevolutionCoordinates[RunevolutionCoordSet][ RunevolutionXOffset[i] ][2]
        else
            -- no exact match, so get two closest coord pairs and interpolate position
            local up = math.ceil( RunevolutionXOffset[i] )
            local down = math.floor( RunevolutionXOffset[i] )
            local x1,y1,x2,y2
            if RunevolutionCoordinates[RunevolutionCoordSet][ up ] then
                x1 = RunevolutionCoordinates[RunevolutionCoordSet][ up ][1]
                y1 = RunevolutionCoordinates[RunevolutionCoordSet][ up ][2]
            else
                x1 = nil
                y1 = nil
            end
            if RunevolutionCoordinates[RunevolutionCoordSet][ down ] then
                x2 = RunevolutionCoordinates[RunevolutionCoordSet][ down ][1]
                y2 = RunevolutionCoordinates[RunevolutionCoordSet][ down ][2]
            else
                x2 = nil
                y2 = nil
            end
            if x1 and x2 then -- interpolate
                local dmax = abs( up - down )
                local d = abs( up - RunevolutionXOffset[i] )
                local p = d / dmax -- proportion of distance from up offset
                x = p*x2 + (1-p)*x1 -- if on up offset, x = x1
                y = p*y2 + (1-p)*y2
            else
                -- at least one coordinate missing, so use the one we have by itself
                if x1 then
                    x = x1
                    y = y1
                else
                    x = x2
                    y = y2
                end
            end
        end
        if not x then x=-1 end
        if not y then y=0 end
        RunevolutionFrames[i]:ClearAllPoints()
        RunevolutionFrames[i]:SetPoint( "CENTER" , RunevolutionData.loc["x"] + x*RunevolutionData.runesize , RunevolutionData.loc["y"] + (y)*RunevolutionData.runesize )
    end
    RunevolutionFrameRP:ClearAllPoints()
    RunevolutionFrameRP:SetPoint( "CENTER" , RunevolutionData.loc["x"] , RunevolutionData.loc["y"] )
    
    RunevolutionFrameMark:ClearAllPoints()
    RunevolutionFrameMark:SetPoint( "CENTER" , RunevolutionData.loc["x"] - RunevolutionData.runesize/2 , RunevolutionData.loc["y"] )
    
end


function Runevolution_AttemptReset()
    Runevolution_UpdateRuneTypes()
    local j = -1
    for i = 1,6 do
        local _,_,ready = GetRuneCooldown(i)
        if ready then
            RunevolutionXOffset[i] = j
            j = j - 1
        else
            RunevolutionXOffset[i] = RunevolutionData.cooldowndistance
        end
    end
    Runevolution_UpdateRunePositions()
end

function Runevolution_Sort( t )
    -- takes table t with rune indexes as keys and ready times as values
    -- returns new table with just indexes, in order of ready times
    local nt = {}
    local n = 0
    for k,v in pairs( t ) do
        n = n + 1
        nt[k] = n
    end
    
    local fChange = true
    local temp
    while fChange do
        fChange = false
        for k,v in pairs( t ) do
            for kk,vv in pairs( t ) do
                if k ~= kk then
                    if v > vv then
                        if nt[k] > nt[kk] then
                            temp = nt[k]
                            nt[k] = nt[kk]
                            nt[kk] = temp
                            fChange = true
                        end
                    elseif v < vv then
                        if nt[k] < nt[kk] then
                            temp = nt[k]
                            nt[k] = nt[kk]
                            nt[kk] = temp
                            fChange = true
                        end
                    end
                end
            end
        end
    end -- while
    return nt
end

local RunevolutionRPPulsedFlag = false
runevreadyrunes = {}
function Runevolution_OnUpdate(elapsed)

    local _,engClass = UnitClass("player")
    if engClass ~= "DEATHKNIGHT" then
        return
    end
    
    -- first, process any pending rune update events
    
    --while RunevolutionRuneUpdateQueue[1] do
    --    local anEvent = RunevolutionRuneUpdateQueue[1]
        --Runevolution_RUNE_POWER_UPDATE( anEvent[1] , anEvent[2] )
    --    table.remove( RunevolutionRuneUpdateQueue , 1 )
    --end
    
    -- new rune updating engine, just uses continuous updating, ignoring events
    local readycount = 0
    local readyrunes = {}
    for i = 1,6 do
        local start,duration,ready = GetRuneCooldown(i)
        if ready then
            -- ready rune, so count
            readycount = readycount + 1
            if not RunevolutionTimeReady[i] then
                RunevolutionTimeReady[i] = readycount
            end
            readyrunes[ i ] = RunevolutionTimeReady[i]
            if RunevolutionPulseList[ i ] == -1 then
                RunevolutionPulseList[ i ] = RunevolutionPulseDuration
            end
            RunevolutionYOffset[ i ] = 0
        else
            -- cooling rune, so mark it so and store the time it'll be ready
            RunevolutionTimeReady[i] = start + duration
            RunevolutionXOffset[i] = 10
            RunevolutionPulseList[ i ] = -1 -- mark that it needs a pulse, once it comes off cooldown
        end
    end
    -- now sort the ready runes by the times they came off cooldown
    readyrunes = Runevolution_Sort( readyrunes )
    local n = 0
    for k,v in pairs( readyrunes ) do
        n = n + 1
        RunevolutionXOffset[ k ] = -v
    end
    Runevolution_UpdateRunePositions()
    runevreadyrunes = readyrunes
    Runevolution_UpdateRuneTypes()
    
    -- now do timing updates
    
    local fAnyReady = false
    local fAnyCooldown = false
    for i = 1,6 do
        if RunevolutionXOffset[i] > 0 or RunevolutionData.static then
            -- rune possibly on cooldown
            local start,duration,ready = GetRuneCooldown(i)
            if not ready then
            
                fAnyCooldown = true
                local remaining = duration - ( GetTime() - start )
                
                if not RunevolutionData.static then
                    -- drift
                    RunevolutionXOffset[i] = remaining/10 * RunevolutionData.cooldowndistance
                    --RunevolutionFrames[i]:ClearAllPoints()
                    --RunevolutionFrames[i]:SetPoint( "CENTER" , RunevolutionData.loc["x"] + RunevolutionXOffset[i]*RunevolutionData.runesize , RunevolutionData.loc["y"] + RunevolutionYOffset[i]*RunevolutionData.runesize )
                else
                    -- static mode
                    local thetext = string.format( "%d" , remaining )
                    local rtype = GetRuneType( i )
                    local color = "FFFFFF"
                    if rtype == 1 then
                        -- blood
                        color = "FF0000"
                    elseif rtype == 2 then
                        -- unholy
                        color = "008000"
                    elseif rtype == 3 then
                        -- frost
                        color = "00FFFF"
                    elseif rtype == 4 then
                        -- death
                        color = "CD1A16"
                    end
                    RunevolutionFrames[i].text:SetText( "|cFF"..color..thetext )
                    RunevolutionFrames[i].text:Show()
                end
                
            else
                -- rune marked for cooldown, but somehow is ready now
                if not RunevolutionData.static then Runevolution_AttemptReset() end
                RunevolutionFrames[i].text:Hide()
                fAnyReady = true
            end
        else
            -- ready rune
            fAnyReady = true
        end
    end -- for i = 1,6
    
    -- finally review and position cooling runes
    -- if 2 or 3 are close, need to stagger them vertically
    for i=1,6 do
        if RunevolutionXOffset[i] > 0 then
            local closerunes = {}
            for j=1,6 do
                if RunevolutionXOffset[j] > 0 and i ~= j then
                    if abs( RunevolutionXOffset[i] - RunevolutionXOffset[j] ) < 1/3 then
                        table.insert( closerunes , j )
                    end
                end
            end
            if closerunes[1] then
                if #closerunes == 2 then
                    --reprint( i.."/"..closerunes[1].."/"..closerunes[2] )
                    RunevolutionYOffset[ i ] = 0
                    RunevolutionYOffset[ closerunes[1] ] = -0.8
                    RunevolutionYOffset[ closerunes[2] ] = 0.8
                elseif #closerunes == 1 then
                    RunevolutionYOffset[ i ] = 0.4
                    RunevolutionYOffset[ closerunes[1] ] = -0.4
                end
            end
        end
    end
    for i = 1,6 do
        if RunevolutionXOffset[i] > 0 then
            RunevolutionFrames[i]:ClearAllPoints()
            RunevolutionFrames[i]:SetPoint( "CENTER" , RunevolutionData.loc["x"] + RunevolutionXOffset[i]*RunevolutionData.runesize , RunevolutionData.loc["y"] + RunevolutionYOffset[i]*RunevolutionData.runesize )
        end
    end
    
    if fAnyReady or RunevolutionData.static then
        RunevolutionFrameMark:Hide()
    else
        RunevolutionFrameMark:Show()
    end
    
    -- pulse animation
    local anypulse = false
    for i = 1,6 do
        if RunevolutionPulseList[i] > 0 then
            anypulse = true
            RunevolutionPulseList[i] = RunevolutionPulseList[i] - elapsed
            if RunevolutionPulseList[i] < 0 then RunevolutionPulseList[i] = 0 end
            local zoom = 1 + RunevolutionPulseList[i]/RunevolutionPulseDuration * (RunevolutionPulseFactor - 1)
            RunevolutionFrames[i]:SetWidth( RunevolutionData.runesize * zoom )
            RunevolutionFrames[i]:SetHeight( RunevolutionData.runesize * zoom )
            
            RunevolutionFrames[i+6]:SetWidth( RunevolutionData.runesize * zoom )
            RunevolutionFrames[i+6]:SetHeight( RunevolutionData.runesize * zoom )
        end
    end
    if RunevolutionPulseList['rp'] > 0 then
        anypulse = true
        RunevolutionPulseList['rp'] = RunevolutionPulseList['rp'] - elapsed
        if RunevolutionPulseList['rp'] < 0 then 
            RunevolutionPulseList['rp'] = 0
        end
        local zoom = 1 + RunevolutionPulseList['rp']/RunevolutionPulseDuration * (RunevolutionPulseFactor - 1)
        RunevolutionFrameRPText:SetFont("Fonts\\FRIZQT__.TTF", zoom * RunevolutionData.runesize/1.5 , "OUTLINE")
    end
    
    -- update runic power
    local rp = UnitPower("player")
    if rp < UnitPowerMax("player") then
        RunevolutionFrameRPText:SetText( "|cFF00B8F5"..rp )
        RunevolutionRPPulsedFlag = false
    else
        RunevolutionFrameRPText:SetText( "|cFFDD55FF"..rp ) -- purple at max runic power
        -- check for pulse animation
        if RunevolutionPulseList['rp'] == 0 and not RunevolutionRPPulsedFlag then
            -- animation not running, so start it
            RunevolutionPulseList['rp'] = RunevolutionPulseDuration
            RunevolutionRPPulsedFlag = true
        end
    end
    RunevolutionFrameRPText:ClearAllPoints()
    local maxleft = -1
    for i = 1,6 do
        if RunevolutionXOffset[i] < maxleft then
            maxleft = RunevolutionXOffset[i]
        end
    end
    if RunevolutionData.fix_rp then
        RunevolutionFrameRPText:SetPoint("CENTER" , RunevolutionFrameRP , "CENTER" ,(-7)*RunevolutionData.runesize , 0 )
    else
        RunevolutionFrameRPText:SetPoint("CENTER" , RunevolutionFrameRP , "CENTER" ,(maxleft-1)*RunevolutionData.runesize , 0 )
    end
    if not RunevolutionData.show_rp then
        RunevolutionFrameRP:Hide()
    else
        RunevolutionFrameRP:Show()
    end
    
    -- update diseases on target
    for k,v in pairs( RunevolutionDiseaseList ) do
        Runevolution_UpdateDiseaseIcon( k )
        
        -- pulse animations
        if RunevolutionPulseList[ k ] > 0 then
            anypulse = true
            RunevolutionPulseList[ k ] = RunevolutionPulseList[ k ] - elapsed
            if RunevolutionPulseList[k] < 0 then RunevolutionPulseList[k] = 0 end
            local zoom = 1 + RunevolutionPulseList[k]/RunevolutionPulseDuration * (RunevolutionPulseFactor - 1)
            RunevolutionFrames[k]:SetWidth( RunevolutionData.runesize * zoom * 0.8 )
            RunevolutionFrames[k]:SetHeight( RunevolutionData.runesize * zoom * 0.8 )
        end
        
        -- flashing alpha disease animations
        -- uses sin-wave shape
        if RunevolutionFlashList[ k ] then
            if RunevolutionFrames[ k ]:IsShown() then
                local rate = 3
                local y = math.sin( GetTime()*rate ) * 0.35 + 0.65  -- scaled to 0.3,1 interval
                RunevolutionFrames[k]:SetAlpha( y )
            end
        end
    end
    
    -- Howling Blast timer
    if RunevolutionHBtimer > 0 then
        RunevolutionHBtimer = RunevolutionHBtimer - elapsed
        if RunevolutionHBtimer < 0 then RunevolutionHBtimer = 0 end
        if RunevolutionHBtimer > 0 then
            RunevolutionFrames["Howling Blast"].text:SetText( "|cFFFFFFFF"..string.format( "%d" , RunevolutionHBtimer ).."s" )
            RunevolutionFrames["Howling Blast"]:Show()
        else
            RunevolutionFrames["Howling Blast"]:Hide()
        end
    end
    
    -- check for out-of-combat
    if InCombatLockdown() or RunevolutionData.show_ooc or fAnyCooldown or anypulse then
        RunevolutionParentContainer:Show()
    else
        RunevolutionParentContainer:Hide()
    end
        
end

function Runevolution_UpdateDiseaseIcon( disease )

    if not RunevolutionData.show_diseases then
        RunevolutionFrames[disease]:Hide()
        return
    end
    
    if RunevolutionDiseaseList[ disease ].presence then
        local _, _, active, _ = GetShapeshiftFormInfo( RunevolutionDiseaseList[ disease ].presence )
        if not active then
            if RunevolutionFrames[disease]:IsShown() then
                RunevolutionFrames[disease]:Hide()
                RunevolutionXOffset[disease] = 0 -- reset position
                -- reorder all the existing spells that are ready
                for k,v in pairs( RunevolutionDiseaseList ) do
                    if k ~= disease and RunevolutionXOffset[k] < 0 then
                        RunevolutionXOffset[k] = 1
                    end
                end
            end
            return
        end
    end

    -- fetch target and type info
    local dtype = RunevolutionDiseaseList[ disease ].type
    local dunit = RunevolutionDiseaseList[ disease ].unit

    if UnitExists( dunit ) then
        
        local expirationTime, isMine = Runevolution_GetExpireTime( disease , dunit , dtype )
        
        if expirationTime == 0 then expirationTime = nil end
        
        if not isMine and not RunevolutionData.show_all_diseases then
            expirationTime = nil -- don't show anything if disease isn't player's and setting off
        end
        
        if expirationTime then
            -- target has debuff, so update position
            local remaining = expirationTime - ( GetTime() )
            --if remaining/10 <= 1 then
            --    RunevolutionXOffset[disease] = remaining/10 * RunevolutionData.cooldowndistance
            --    RunevolutionFrames[disease].text:Hide()
            if true then
                -- more than 10 seconds to go, so fix under ready runes and label with duration
                
                local fstring = string.format( "%d" , remaining ).."s"
                if remaining > 60 then
                    fstring = string.format( "%d" , remaining/60 + 1 ).."m"
                end
                RunevolutionFrames[disease].text:SetText( "|cFFFFFFFF"..fstring )
                RunevolutionFrames[disease].text:Show()
                
                local top = true
                if not RunevolutionData.diseases_up then
                    top = false
                end
                
                if dtype == "spell" then
                    top = not top -- put cooldowns on opposite side from diseases
                end
                
                if top then
                    RunevolutionYOffset[disease] = 1
                    RunevolutionFrames[disease].text:ClearAllPoints()
                    RunevolutionFrames[disease].text:SetPoint("BOTTOM" , RunevolutionFrames[disease] , "TOP" , 0 , 0)
                else
                    RunevolutionYOffset[disease] = -1
                    RunevolutionFrames[disease].text:ClearAllPoints()
                    RunevolutionFrames[disease].text:SetPoint("TOP" , RunevolutionFrames[disease] , "BOTTOM" , 0 , 0)
                end
                
                -- fade, if debuff not ours
                if dtype == "debuff" and not isMine then
                    RunevolutionFrames[disease]:SetAlpha( 0.5 )
                    RunevolutionFrames[disease].text:Hide()
                else
                    RunevolutionFrames[disease]:SetAlpha( 1 )
                    RunevolutionFrames[disease].text:Show()
                end
                
                -- reorder, if necessary
                if dtype ~= "spell" then
                    if RunevolutionXOffset[disease] == 0 then
                        RunevolutionXOffset[disease] = -1
                        for v,vv in pairs( RunevolutionDiseaseList ) do
                            if v ~= disease and vv.type ~= "spell" then
                                -- another disease/buff possibly up, so stack them
                                if RunevolutionXOffset[ v ] < 0 then
                                    RunevolutionXOffset[disease] = RunevolutionXOffset[disease] - 1
                                end
                            end
                        end
                    end -- if offset == 0
                    
                else
                    -- "spell"
                    if RunevolutionXOffset[disease] < 0 then -- need to position it
                        local oldpos = RunevolutionXOffset[disease]
                        RunevolutionXOffset[disease] = 0.01
                        for v,vv in pairs( RunevolutionDiseaseList ) do
                            if v ~= disease and vv.type == "spell" then
                                -- another disease/buff possibly up, so stack them
                                if RunevolutionXOffset[ v ] > 0 then
                                    RunevolutionXOffset[disease] = RunevolutionXOffset[disease] + 1
                                end
                                -- shift any ready spells to right, if they were left of this one
                                if RunevolutionXOffset[ v ] < oldpos then
                                    RunevolutionXOffset[ v ] = RunevolutionXOffset[ v ] + 1
                                end
                            end
                        end
                    end -- if offset < 0
                    
                end
                
                -- pulse when close to expire
                local startflash = 6
                if remaining < startflash and not RunevolutionFlashList[ disease ] then
                    RunevolutionFlashList[ disease ] = 0 -- initiate animation
                elseif remaining > startflash then
                    RunevolutionFlashList[ disease ] = nil
                end
                
            end
            RunevolutionFrames[disease]:ClearAllPoints()
        RunevolutionFrames[disease]:SetPoint( "CENTER" , RunevolutionData.loc["x"] + RunevolutionXOffset[disease]*RunevolutionData.runesize , RunevolutionData.loc["y"] + RunevolutionYOffset[disease]*RunevolutionData.runesize )
            RunevolutionFrames[disease]:Show()
            
        else
            -- no expiration time, but check if a spell/ability. if so, we show it on left when it's ready for use
            RunevolutionFlashList[ disease ] = nil -- make sure flash is turned off
            if dtype == "spell" then
                
                -- reorder, if necessary
                if RunevolutionXOffset[disease] >= 0 then -- needs to come off cooling position
                    local oldpos = RunevolutionXOffset[disease]
                    RunevolutionXOffset[disease] = -1
                    for v,vv in pairs( RunevolutionDiseaseList ) do
                        if v ~= disease and vv.type == "spell" then
                            -- another cooldown possibly up, so stack them
                            if RunevolutionXOffset[ v ] < 0 then
                                RunevolutionXOffset[disease] = RunevolutionXOffset[disease] - 1
                            end
                            -- shift other spells left, if still on cooldown
                            if RunevolutionXOffset[ v ] > oldpos then
                                RunevolutionXOffset[ v ] = RunevolutionXOffset[ v ] - 1
                            end
                        end
                    end
                    -- init pulse animation
                    RunevolutionPulseList[ disease ] = RunevolutionPulseDuration
                end -- if offset >= 0
                
                RunevolutionFrames[disease]:ClearAllPoints()
                RunevolutionFrames[disease]:SetPoint( "CENTER" , RunevolutionData.loc["x"] + RunevolutionXOffset[disease]*RunevolutionData.runesize , RunevolutionData.loc["y"] + RunevolutionYOffset[disease]*RunevolutionData.runesize )
                RunevolutionFrames[disease]:Show()
                RunevolutionFrames[disease].text:Hide()
                
            else
                -- mark that this disease is not positioned and hide it
                if RunevolutionXOffset[disease] < 0 then
                    -- reposition other diseases
                    for k,v in pairs( RunevolutionDiseaseList ) do
                        if k ~= disease and v.type ~= "spell" and RunevolutionXOffset[k] < RunevolutionXOffset[disease] then
                            RunevolutionXOffset[k] = RunevolutionXOffset[k] + 1
                        end
                    end
                end
                RunevolutionXOffset[disease] = 0
                RunevolutionFrames[disease]:Hide()
                
            end
        end
        
    else
        RunevolutionFrames[disease]:Hide()
    end
    
end

function Runevolution_GetExpireTime( disease , dunit , dtype )
    local start , name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable
    
    expirationTime = nil
    isMine = nil
    
    if dtype == "debuff" then
        name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable  =  UnitDebuff( dunit , disease )
    end
    
    if dtype == "spell" then
    
        start, duration, _ = GetSpellCooldown( disease , BOOKTYPE_SPELL )
        
        if disease == "Blood Tap" then
            --reprint( start.."/"..duration )
        end
        
        if start then
            expirationTime = start + duration
            isMine = true
        end
        if start == 0 then
            expirationTime = nil
        end
        
        -- finally check if cooldown from global cooldown only, by checking Blood Strike
        local bsStart = GetSpellCooldown( "Blood Strike" , BOOKTYPE_SPELL )
        
        --reprint( tostring(bsStart).."/"..tostring(start) )
        
        if bsStart then
            if bsStart == start then
                -- global cooldown triggered
                expirationTime = nil
            end
        end
    end
    
    if dtype == "buff" then
        name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable = UnitBuff( dunit , disease )
    end
    
    return expirationTime, isMine
end